@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: ObjectFactory.java,v 1.2 2004/02/23 10:29:34 aruny Exp $
+ * $Id: ObjectFactory.java,v 1.5 2004/08/17 18:14:49 jycli Exp $
  */
 
 package org.apache.xalan.lib;
@@ -43,7 +43,7 @@ import java.io.InputStreamReader;
  * class and modified to be used as a general utility for creating objects 
  * dynamically.
  *
- * @version $Id: ObjectFactory.java,v 1.2 2004/02/23 10:29:34 aruny Exp $
+ * @version $Id: ObjectFactory.java,v 1.5 2004/08/17 18:14:49 jycli Exp $
  */
 class ObjectFactory {
 
@@ -291,6 +291,7 @@ class ObjectFactory {
 
             synchronized (ObjectFactory.class) {
                 boolean loadProperties = false;
+                FileInputStream fis = null;
                 try {
                     // file existed last time
                     if(fLastModified >= 0) {
@@ -315,35 +316,52 @@ class ObjectFactory {
                         // must never have attempted to read xalan.properties
                         // before (or it's outdeated)
                         fXalanProperties = new Properties();
-                        FileInputStream fis =
-                                         ss.getFileInputStream(propertiesFile);
+                        fis = ss.getFileInputStream(propertiesFile);
                         fXalanProperties.load(fis);
-                        fis.close();
                     }
-	            } catch (Exception x) {
-	                fXalanProperties = null;
-	                fLastModified = -1;
-                        // assert(x instanceof FileNotFoundException
-	                //        || x instanceof SecurityException)
-	                // In both cases, ignore and continue w/ next location
-	            }
+	        } catch (Exception x) {
+	            fXalanProperties = null;
+	            fLastModified = -1;
+                    // assert(x instanceof FileNotFoundException
+	            //        || x instanceof SecurityException)
+	            // In both cases, ignore and continue w/ next location
+	        }
+                finally {
+                    // try to close the input stream if one was opened.
+                    if (fis != null) {
+                        try {
+                            fis.close();
+                        }
+                        // Ignore the exception.
+                        catch (IOException exc) {}
+                    }
+                }	            
             }
             if(fXalanProperties != null) {
                 factoryClassName = fXalanProperties.getProperty(factoryId);
             }
         } else {
+            FileInputStream fis = null;
             try {
-                FileInputStream fis =
-                           ss.getFileInputStream(new File(propertiesFilename));
+                fis = ss.getFileInputStream(new File(propertiesFilename));
                 Properties props = new Properties();
                 props.load(fis);
-                fis.close();
                 factoryClassName = props.getProperty(factoryId);
             } catch (Exception x) {
                 // assert(x instanceof FileNotFoundException
                 //        || x instanceof SecurityException)
                 // In both cases, ignore and continue w/ next location
             }
+            finally {
+                // try to close the input stream if one was opened.
+                if (fis != null) {
+                    try {
+                        fis.close();
+                    }
+                    // Ignore the exception.
+                    catch (IOException exc) {}
+                }
+            }               
         }
         if (factoryClassName != null) {
             debugPrintln("found in " + propertiesFilename + ", value="
@@ -461,9 +479,12 @@ class ObjectFactory {
         //class. Restrict the access to the package classes as specified in java.security policy.
         SecurityManager security = System.getSecurityManager();
         try{
-            if (security != null){
-                security.checkPackageAccess(className);
-             }   
+                if (security != null){
+                    final int lastDot = className.lastIndexOf(".");
+                    String packageName = className;
+                    if (lastDot != -1) packageName = className.substring(0, lastDot);
+                    security.checkPackageAccess(packageName);
+                 }   
         }catch(SecurityException e){
             throw e;
         }
@@ -565,11 +586,18 @@ class ObjectFactory {
             // XXX Does not handle all possible input as specified by the
             // Jar Service Provider specification
             factoryClassName = rd.readLine();
-            rd.close();
         } catch (IOException x) {
             // No provider found
             return null;
         }
+        finally {
+            try {
+                // try to close the reader.
+                rd.close();
+            }
+            // Ignore the exception.
+            catch (IOException exc) {}
+        }          
 
         if (factoryClassName != null &&
             ! "".equals(factoryClassName)) {
@@ -596,7 +624,7 @@ class ObjectFactory {
      */
     static class ConfigurationError 
         extends Error {
-
+                static final long serialVersionUID = -7640369932165775029L;
         //
         // Data
         //
