@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: DTMDocument.java,v 1.15 2004/02/11 17:56:36 minchau Exp $
+ * $Id: DTMDocument.java,v 1.17 2005/01/23 00:16:04 mcnamara Exp $
  */
 
 package org.apache.xalan.lib.sql;
@@ -615,7 +615,7 @@ public class DTMDocument extends DTMDefaultBaseIterators
     catch(IOException ioe)
     {
       ioe.printStackTrace(System.err);
-      System.exit(-1);
+      throw new RuntimeException(ioe.getMessage());
     }
   }
 
@@ -871,8 +871,6 @@ public class DTMDocument extends DTMDefaultBaseIterators
    * @param parm2
    *
    * @throws org.xml.sax.SAXException
-   *
-   * @throws org.xml.sax.SAXException
    */
   public void dispatchToEvents( int parm1, ContentHandler parm2 )throws org.xml.sax.SAXException
   {
@@ -902,8 +900,6 @@ public class DTMDocument extends DTMDefaultBaseIterators
    * @param normalize
    *
    * @throws org.xml.sax.SAXException
-   *
-   * @throws org.xml.sax.SAXException
    */
   public void dispatchCharactersEvents( int nodeHandle, ContentHandler ch, boolean normalize )throws org.xml.sax.SAXException
   {
