@@ -14,40 +14,35 @@
  * limitations under the License.
  */
 /*
- * $Id: XConnection.java,v 1.29 2004/02/11 17:56:36 minchau Exp $
+ * $Id: XConnection.java,v 1.34 2005/07/28 00:26:16 johng Exp $
  */
 package org.apache.xalan.lib.sql;
 
-import java.math.BigDecimal;
 import java.sql.Connection;
-import java.sql.Date;
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
-import java.sql.Statement;
-import java.sql.Time;
-import java.sql.Timestamp;
-import java.util.Enumeration;
 import java.util.Properties;
 import java.util.StringTokenizer;
 import java.util.Vector;
 
+import javax.xml.transform.ErrorListener;
+import javax.xml.transform.TransformerException;
+
 import org.apache.xalan.extensions.ExpressionContext;
 import org.apache.xml.dtm.DTM;
+import org.apache.xml.dtm.DTMIterator;
 import org.apache.xml.dtm.DTMManager;
 import org.apache.xml.dtm.ref.DTMManagerDefault;
+import org.apache.xml.dtm.ref.DTMNodeIterator;
+import org.apache.xml.dtm.ref.DTMNodeProxy;
 import org.apache.xpath.XPathContext;
 import org.apache.xpath.objects.XBooleanStatic;
-
+import org.apache.xpath.objects.XNodeSet;
 import org.w3c.dom.Element;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 
-import javax.xml.transform.ErrorListener;
-import javax.xml.transform.TransformerException;
-
 /**
  * An XSLT extension that allows a stylesheet to
  * access JDBC data.
@@ -197,17 +192,20 @@ public class XConnection
   {
   }
 
-  // The original constructors will be kept around for backwards
-  // compatibility. Future Stylesheets should use the approaite
-  // connect method to receive full error information.
-  //
   /**
-   * @param exprContext
-   * @param ConnPoolName
+   * Constructs a new XConnection and attempts to connect to a datasource as
+   * defined in the
+   * <code>connect(ExpressionContext exprContext, String connPoolName)</code>
+   * method.
+   * <code>org.apache.xalan.lib.sql.ConnectionPool</code> or a JNDI datasource.
+   *
+   * @param exprContext Context automatically passed from the XSLT sheet.
+   * @param name The name of the ConnectionPool or the JNDI DataSource path.
+   *
    */
-  public XConnection( ExpressionContext exprContext, String ConnPoolName )
+  public XConnection( ExpressionContext exprContext, String connPoolName )
   {
-    connect(exprContext, ConnPoolName);
+    connect(exprContext, connPoolName);
   }
 
   /**
@@ -252,32 +250,63 @@ public class XConnection
     connect(exprContext, driver, dbURL, protocolElem);
   }
 
-
   /**
-   * Create an XConnection using the name of an existing Connection Pool
-   * @param exprContext
-   * @param ConnPoolName
-   *
-   */
-  public XBooleanStatic connect( ExpressionContext exprContext, String ConnPoolName )
-  {
-    try
-    {
-      m_ConnectionPool = m_PoolMgr.getPool(ConnPoolName);
+    * Returns an XConnection from either a user created
+    * <code>org.apache.xalan.lib.sql.ConnectionPool</code> or a JNDI datasource.
 
-      if (m_ConnectionPool == null)
-        throw new java.lang.IllegalArgumentException("Invalid Pool Name");
+    * 
+    * This method first tries to resolve the passed name against
+    * <code>ConnectionPool</code>s registered with
+    * <code>ConnectionPoolManager</code>.
+    * If that fails, it attempts to find the name as a JNDI DataSource path.
+    *
+    * @param exprContext Context automatically passed from the XSLT sheet.
+    * @param name The name of the ConnectionPool or the JNDI DataSource path.
+    *
+    */
+   public XBooleanStatic connect( ExpressionContext exprContext, String name )
+   {
+     try
+     {
+       m_ConnectionPool = m_PoolMgr.getPool(name);
 
-      m_IsDefaultPool = false;
-      return new XBooleanStatic(true);
-    }
-    catch (Exception e)
-    {
-      setError(e, exprContext);
-      return new XBooleanStatic(false);
-    }
+       if (m_ConnectionPool == null)
+       {
+         //Try to create a jndi source with the passed name
+         ConnectionPool pool = new JNDIConnectionPool(name);
+        
+         if (pool.testConnection())
+         {
+          
+           //JNDIConnectionPool seems good, so register it with the pool manager.
+           //Once registered, it will show up like other named ConnectionPool's,
+           //so the m_PoolMgr.getPool(name) method (above) will find it.
+           m_PoolMgr.registerPool(name, pool);
+           m_ConnectionPool = pool;
+          
+           m_IsDefaultPool = false;
+           return new XBooleanStatic(true);
+         }
+         else
+         {
+           throw new IllegalArgumentException(
+               "Invalid ConnectionPool name or JNDI Datasource path: " + name);
+         }
+       }
+       else
+       {
+         m_IsDefaultPool = false;
+         return new XBooleanStatic(true);
+       }
+     }
+     catch (Exception e)
+     {
+       setError(e, exprContext);
+       return new XBooleanStatic(false);
+     }
+
+   }
 
-  }
 
   /**
    * Create an XConnection object with just a driver and database URL.
@@ -677,7 +706,7 @@ public class XConnection
           setError(e, doc, doc.checkWarnings());
         }
 
-        doc.close();
+        doc.close(m_IsDefaultPool);
         doc = null;
       }
     }
@@ -767,7 +796,9 @@ public class XConnection
           setError(e, doc, doc.checkWarnings());
         }
 
-        doc.close();
+        // If we are using the Default Connection Pool, then
+        // force the connection pool to flush unused connections.
+        doc.close(m_IsDefaultPool);
         doc = null;
       }
     }
@@ -780,6 +811,27 @@ public class XConnection
     return doc;
   }
 
+  /**
+   * The purpose of this routine is to force the DB cursor to skip forward
+   * N records. You should call this function after [p]query to help with
+   * pagination. i.e. Perfrom your select, then skip forward past the records
+   * you read previously.
+   * 
+   * @param exprContext
+   * @param o
+   * @param value
+   */
+  public void skipRec( ExpressionContext exprContext, Object o, int value )
+  {
+    SQLDocument sqldoc = null;
+    DTMNodeIterator nodei = null;
+      
+    sqldoc = locateSQLDocument( exprContext, o);
+    if (sqldoc != null) sqldoc.skip(value);
+  }
+
+  
+
   private void addTypeToData(String typeInfo)
   {
       int indx;
@@ -1056,7 +1108,9 @@ public class XConnection
       SQLDocument d = (SQLDocument) m_OpenSQLDocuments.elementAt(0);
       try
       {
-        d.close();
+        // If we are using the Default Connection Pool, then
+        // force the connection pool to flush unused connections.
+        d.close(m_IsDefaultPool);
       }
       catch (Exception se ) {}
 
@@ -1076,29 +1130,73 @@ public class XConnection
   /**
    * Close the connection to the data source. Only close the connections
    * for a single document.
-   * @param sqldoc
    *
    * @throws SQLException
    */
-  public void close( SQLDocument sqldoc )throws SQLException
+
+  public void close(ExpressionContext exprContext, Object doc) throws SQLException 
   {
     if (DEBUG)
-      System.out.println("Entering XConnection.close(" + sqldoc + ")");
+        System.out.println("Entering XConnection.close(" + doc + ")");
 
-    int size = m_OpenSQLDocuments.size();
-
-    for(int x=0; x<size; x++)
+    SQLDocument sqlDoc = locateSQLDocument(exprContext, doc);
+    if (sqlDoc != null)
     {
-      SQLDocument d = (SQLDocument) m_OpenSQLDocuments.elementAt(x);
-      if (d == sqldoc)
-      {
-        d.close();
-        m_OpenSQLDocuments.removeElementAt(x);
-      }
-    }
+      // If we are using the Default Connection Pool, then
+      // force the connection pool to flush unused connections.
+      sqlDoc.close(m_IsDefaultPool);
+      m_OpenSQLDocuments.remove(sqlDoc);
+    } 
   }
 
 
+  /**
+   * When an SQL Document is returned as a DTM object, the XSL variable is actually 
+   * assigned as a DTMIterator. This is a helper function that will allow you to get
+   * a reference to the original SQLDocument from the iterator.
+   * 
+   * Original code submitted by 
+   *  Moraine Didier mailto://didier.moraine@winterthur.be
+   * @param doc
+   * @return
+   */
+  private SQLDocument locateSQLDocument(ExpressionContext exprContext, Object doc)
+  {
+    try
+    {
+      if (doc instanceof DTMNodeIterator)
+      {
+        DTMNodeIterator dtmIter = (DTMNodeIterator)doc;
+        try
+        {
+          DTMNodeProxy root = (DTMNodeProxy)dtmIter.getRoot();
+          return (SQLDocument) root.getDTM();
+        }
+        catch (Exception e)
+        {
+          XNodeSet xNS = (XNodeSet)dtmIter.getDTMIterator();
+          DTMIterator iter = (DTMIterator)xNS.getContainedIter();
+          DTM dtm = iter.getDTM(xNS.nextNode());
+          return (SQLDocument)dtm;
+        }
+      }
+
+/*
+      XNodeSet xNS = (XNodeSet)dtmIter.getDTMIterator();
+      OneStepIterator iter = (OneStepIterator)xNS.getContainedIter();
+      DTMManager aDTMManager = (DTMManager)iter.getDTMManager();
+      return (SQLDocument)aDTMManager.getDTM(xNS.nextNode());
+*/
+      setError(new Exception("SQL Extension:close - Can Not Identify SQLDocument"), exprContext);    
+      return null;  
+    }
+    catch(Exception e)
+    {
+      setError(e, exprContext);
+      return null;
+    }
+  }
+  
   /**
    * @param exprContext
    * @param excp
@@ -1158,6 +1256,7 @@ public class XConnection
       ErrorListener listen = expr.getErrorListener();
       if ( listen != null && excp != null )
       {
+        
         listen.warning(
           new TransformerException(excp.toString(),
           expr.getXPathContext().getSAXLocator(), excp));
@@ -1168,8 +1267,6 @@ public class XConnection
 
   /**
    * Set an error and/or warning on this connection.
-   * @param feature The name of the feature being set, currently supports (streaming, inline-variables, multiple-results, cache-statements, default-pool-enabled).
-   * @param setting The new setting for the specified feature, currently "true" is true and anything else is false.
    *
    */
   public void setError(Exception excp, SQLDocument doc, SQLWarning warn)
