@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: StylesheetHandler.java,v 1.64 2004/02/17 22:10:49 minchau Exp $
+ * $Id: StylesheetHandler.java,v 1.70 2005/06/22 19:07:10 mkwan Exp $
  */
 package org.apache.xalan.processor;
 
@@ -65,18 +65,26 @@ public class StylesheetHandler extends DefaultHandler
 {
 
 
-  static {
-      Function func = new org.apache.xalan.templates.FuncDocument();
-
-      FunctionTable.installFunction("document", func);
-
-      // func = new org.apache.xalan.templates.FuncKey();
-      // FunctionTable.installFunction("key", func);
-      func = new org.apache.xalan.templates.FuncFormatNumb();
-
-      FunctionTable.installFunction("format-number", func);
-
-  }
+  /**
+   * The function table of XPath and XSLT;
+   */
+  private FunctionTable m_funcTable = new FunctionTable();
+  
+  /**
+   * The flag for the setting of the optimize feature;
+   */
+  private boolean m_optimize = true;
+  
+  /**
+   * The flag for the setting of the incremental feature;
+   */
+  private boolean m_incremental = false;
+  
+  /**
+   * The flag for the setting of the source_location feature;
+   */
+  private boolean m_source_location = false;
+  
   /**
    * Create a StylesheetHandler object, creating a root stylesheet
    * as the target.
@@ -89,9 +97,24 @@ public class StylesheetHandler extends DefaultHandler
   public StylesheetHandler(TransformerFactoryImpl processor)
           throws TransformerConfigurationException
   {
+    Class func = org.apache.xalan.templates.FuncDocument.class;
+    m_funcTable.installFunction("document", func);
 
+    // func = new org.apache.xalan.templates.FuncKey();
+    // FunctionTable.installFunction("key", func);
+    func = org.apache.xalan.templates.FuncFormatNumb.class;
+
+    m_funcTable.installFunction("format-number", func);
+
+    m_optimize =((Boolean) processor.getAttribute(
+            TransformerFactoryImpl.FEATURE_OPTIMIZE)).booleanValue();
+    m_incremental = ((Boolean) processor.getAttribute(
+            TransformerFactoryImpl.FEATURE_INCREMENTAL)).booleanValue();
+    m_source_location = ((Boolean) processor.getAttribute(
+            TransformerFactoryImpl.FEATURE_SOURCE_LOCATION)).booleanValue();
     // m_schema = new XSLTSchema();
     init(processor);
+    
   }
 
   /**
@@ -126,7 +149,8 @@ public class StylesheetHandler extends DefaultHandler
           throws javax.xml.transform.TransformerException
   {
     ErrorListener handler = m_stylesheetProcessor.getErrorListener();
-    XPath xpath = new XPath(str, owningTemplate, this, XPath.SELECT, handler);
+    XPath xpath = new XPath(str, owningTemplate, this, XPath.SELECT, handler, 
+            m_funcTable);
     // Visit the expression, registering namespaces for any extension functions it includes.
     xpath.callVisitors(xpath, new ExpressionVisitor(getStylesheetRoot()));
     return xpath;
@@ -146,7 +170,8 @@ public class StylesheetHandler extends DefaultHandler
           throws javax.xml.transform.TransformerException
   {
     ErrorListener handler = m_stylesheetProcessor.getErrorListener();
-    XPath xpath = new XPath(str, owningTemplate, this, XPath.MATCH, handler);
+    XPath xpath = new XPath(str, owningTemplate, this, XPath.MATCH, handler, 
+        m_funcTable);
     // Visit the expression, registering namespaces for any extension functions it includes.
     xpath.callVisitors(xpath, new ExpressionVisitor(getStylesheetRoot()));
     return xpath;    
@@ -533,8 +558,6 @@ public class StylesheetHandler extends DefaultHandler
   /**
    * Receive notification of the start of an element.
    *
-   * @param name The element type name.
-   *
    * @param uri The Namespace URI, or an empty string.
    * @param localName The local name (without prefix), or empty string if not namespace processing.
    * @param rawName The qualified name (with prefix).
@@ -610,9 +633,6 @@ public class StylesheetHandler extends DefaultHandler
   /**
    * Receive notification of the end of an element.
    *
-   * @param name The element type name.
-   * @param attributes The specified or defaulted attributes.
-   *
    * @param uri The Namespace URI, or an empty string.
    * @param localName The local name (without prefix), or empty string if not namespace processing.
    * @param rawName The qualified name (with prefix).
@@ -1086,28 +1106,31 @@ public class StylesheetHandler extends DefaultHandler
    *
    * @return The TransformerFactoryImpl that owns this handler.
    */
-  TransformerFactoryImpl getStylesheetProcessor()
+  public TransformerFactoryImpl getStylesheetProcessor()
   {
     return m_stylesheetProcessor;
   }
 
   /**
-   * If {@link #getStylesheetType} returns this value, the current stylesheet
+   * If getStylesheetType returns this value, the current stylesheet
    *  is a root stylesheet.
+   * @xsl.usage internal
    */
-  static final int STYPE_ROOT = 1;
+  public static final int STYPE_ROOT = 1;
 
   /**
-   * If {@link #getStylesheetType} returns this value, the current stylesheet
+   * If getStylesheetType returns this value, the current stylesheet
    *  is an included stylesheet.
+   * @xsl.usage internal
    */
-  static final int STYPE_INCLUDE = 2;
+  public static final int STYPE_INCLUDE = 2;
 
   /**
-   * If {@link #getStylesheetType} returns this value, the current stylesheet
+   * If getStylesheetType returns this value, the current stylesheet
    *  is an imported stylesheet.
+   * @xsl.usage internal
    */
-  static final int STYPE_IMPORT = 3;
+  public static final int STYPE_IMPORT = 3;
 
   /** The current stylesheet type. */
   private int m_stylesheetType = STYPE_ROOT;
@@ -1168,6 +1191,11 @@ public class StylesheetHandler extends DefaultHandler
    */
   public StylesheetRoot getStylesheetRoot()
   {
+    if (m_stylesheetRoot != null){
+        m_stylesheetRoot.setOptimizer(m_optimize);
+        m_stylesheetRoot.setIncremental(m_incremental);
+        m_stylesheetRoot.setSource_location(m_source_location);  		
+    }
     return m_stylesheetRoot;
   }
 
@@ -1263,8 +1291,9 @@ public class StylesheetHandler extends DefaultHandler
    * transition content handlers, create elements, etc.
    *
    * @return The root XSLT Schema, which should never be null.
+   * @xsl.usage internal
    */
-  XSLTSchema getSchema()
+  public XSLTSchema getSchema()
   {
     return m_schema;
   }
@@ -1615,7 +1644,7 @@ public class StylesheetHandler extends DefaultHandler
     while ((version == -1 || version == Constants.XSLTVERSUPPORTED) && elem != null)
     {
       try{
-      version = Double.valueOf(elem.getVersion()).doubleValue();
+      version = Double.valueOf(elem.getXmlVersion()).doubleValue();
       }
       catch (Exception ex)
       {
@@ -1625,12 +1654,33 @@ public class StylesheetHandler extends DefaultHandler
       }
     return (version == -1)? Constants.XSLTVERSUPPORTED : version;
   }
-	/**
-	 * @see PrefixResolver#handlesNullPrefixes()
-	 */
-	public boolean handlesNullPrefixes() {
-		return false;
-	}
+    /**
+     * @see PrefixResolver#handlesNullPrefixes()
+     */
+    public boolean handlesNullPrefixes() {
+        return false;
+    }
+
+    /**
+     * @return Optimization flag
+     */
+    public boolean getOptimize() {
+        return m_optimize;
+    }
+
+    /**
+     * @return Incremental flag
+     */
+    public boolean getIncremental() {
+        return m_incremental;
+    }
+
+    /**
+     * @return Source Location flag
+     */
+    public boolean getSource_location() {
+        return m_source_location;
+    }
 
 }
 
