@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: XSLTErrorResources_pt_BR.java,v 1.2 2004/02/16 19:14:07 minchau Exp $
+ * $Id: XSLTErrorResources_pt_BR.java,v 1.3 2004/12/15 17:35:32 jycli Exp $
  */
 package org.apache.xalan.res;
 
@@ -452,8 +452,13 @@ public class XSLTErrorResources_pt_BR extends ListResourceBundle
 
   // Error messages...
 
-  /** The lookup table for error messages.   */
-  public static final Object[][] contents = {
+  /** Get the lookup table for error messages.   
+   *
+   * @return The int to message lookup table.
+   */
+  public Object[][] getContents()
+  {
+    return new Object[][] {
 
   /** Error message ID that has a null message, but takes in a single object.    */
   {"ER0000" , "{0}" },
@@ -2372,6 +2377,7 @@ public class XSLTErrorResources_pt_BR extends ListResourceBundle
   { "matchPatternIs", "o padr\u00e3o de correspond\u00eancia \u00e9" }
 
   };
+  }
 
   // ================= INFRASTRUCTURE ======================
 
@@ -2400,15 +2406,6 @@ public class XSLTErrorResources_pt_BR extends ListResourceBundle
    * @deprecated  */
   public static final String QUERY_HEADER = "PADR\u00c3O ";
 
-  /**
-   * Get the lookup table.
-   *
-   * @return The int to message lookup table.
-   */
-  public Object[][] getContents()
-  {
-    return contents;
-  }
 
   /**
    *   Return a named ResourceBundle for a particular locale.  This method mimics the behavior
