@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: TransformerIdentityImpl.java,v 1.31 2004/02/23 21:33:14 igorh Exp $
+ * $Id: TransformerIdentityImpl.java,v 1.37 2005/07/05 17:55:44 zongaro Exp $
  */
 package org.apache.xalan.transformer;
 
@@ -22,6 +22,7 @@ import java.io.IOException;
 import java.util.Hashtable;
 import java.util.Properties;
 
+import javax.xml.XMLConstants;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
@@ -47,7 +48,6 @@ import org.apache.xml.serializer.Serializer;
 import org.apache.xml.serializer.SerializerFactory;
 import org.apache.xml.serializer.Method;
 import org.apache.xml.utils.DOMBuilder;
-import org.apache.xml.utils.TreeWalker;
 import org.apache.xml.utils.XMLReaderManager;
 
 import org.w3c.dom.Document;
@@ -81,9 +81,19 @@ public class TransformerIdentityImpl extends Transformer
    * Constructor TransformerIdentityImpl creates an identity transform.
    *
    */
-  public TransformerIdentityImpl()
+  public TransformerIdentityImpl(boolean isSecureProcessing)
   {
     m_outputFormat = new OutputProperties(Method.XML);
+    m_isSecureProcessing = isSecureProcessing;
+  }
+
+  /**
+   * Constructor TransformerIdentityImpl creates an identity transform.
+   *
+   */
+  public TransformerIdentityImpl()
+  {
+    this(false);
   }
 
   /**
@@ -132,6 +142,26 @@ public class TransformerIdentityImpl extends Transformer
     return this;
   }
 
+  /**
+   * Reset the status of the transformer.
+   */
+  public void reset()
+  {
+    m_flushedStartDoc = false;
+    m_foundFirstElement = false;
+    m_outputStream = null;
+    m_params.clear();
+    m_result = null;
+    m_resultContentHandler = null;
+    m_resultDeclHandler = null;
+    m_resultDTDHandler = null;
+    m_resultLexicalHandler = null;
+    m_serializer = null;
+    m_systemID = null;
+    m_URIResolver = null;
+    m_outputFormat = new OutputProperties(Method.XML);
+  }
+
   /**
    * Create a result ContentHandler from a Result object, based
    * on the current OutputProperties.
@@ -166,6 +196,7 @@ public class TransformerIdentityImpl extends Transformer
     {
       DOMResult domResult = (DOMResult) outputTarget;
       Node outputNode = domResult.getNode();
+      Node nextSibling = domResult.getNextSibling();
       Document doc;
       short type;
 
@@ -183,6 +214,15 @@ public class TransformerIdentityImpl extends Transformer
 
           dbf.setNamespaceAware(true);
 
+          if (m_isSecureProcessing)
+          {
+            try
+            {
+              dbf.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
+            }
+            catch (ParserConfigurationException pce) {}
+          }
+
           DocumentBuilder db = dbf.newDocumentBuilder();
 
           doc = db.newDocument();
@@ -198,11 +238,16 @@ public class TransformerIdentityImpl extends Transformer
         ((DOMResult) outputTarget).setNode(outputNode);
       }
 
-      m_resultContentHandler =
+      DOMBuilder domBuilder =
         (Node.DOCUMENT_FRAGMENT_NODE == type)
         ? new DOMBuilder(doc, (DocumentFragment) outputNode)
         : new DOMBuilder(doc, outputNode);
-      m_resultLexicalHandler = (LexicalHandler) m_resultContentHandler;
+      
+      if (nextSibling != null)
+        domBuilder.setNextSibling(nextSibling);
+      
+      m_resultContentHandler = domBuilder;
+      m_resultLexicalHandler = domBuilder;
     }
     else if (outputTarget instanceof StreamResult)
     {
@@ -224,12 +269,18 @@ public class TransformerIdentityImpl extends Transformer
         {
           String fileURL = sresult.getSystemId();
 
-          if (fileURL.startsWith("file:///"))
-          {
-            if (fileURL.substring(8).indexOf(":") >0)
+          if (fileURL.startsWith("file:///")) {
+            if (fileURL.substring(8).indexOf(":") >0) {
               fileURL = fileURL.substring(8);
-            else 
+            } else  {
               fileURL = fileURL.substring(7);
+            }
+          } else if (fileURL.startsWith("file:/")) {
+            if (fileURL.substring(6).indexOf(":") >0) {
+              fileURL = fileURL.substring(6);
+            } else {
+              fileURL = fileURL.substring(5);
+            }
           }
 
           m_outputStream = new java.io.FileOutputStream(fileURL);
@@ -332,8 +383,9 @@ public class TransformerIdentityImpl extends Transformer
                 characters(chars, 0, chars.length);
               }
               else
-              {
-                TreeWalker walker = new TreeWalker(this, new org.apache.xml.utils.DOM2Helper(), m_systemID);
+              { 
+                org.apache.xml.serializer.TreeWalker walker;
+                walker = new org.apache.xml.serializer.TreeWalker(this, m_systemID);
                 walker.traverse(dNode);
               }
             }
@@ -1036,7 +1088,6 @@ public class TransformerIdentityImpl extends Transformer
    *        performed.
    * @param qName The qualified name (with prefix), or the
    *        empty string if qualified names are not available.
-   * @param attributes The specified or defaulted attributes.
    *
    * @throws org.xml.sax.SAXException Any SAX exception, possibly
    *            wrapping another exception.
@@ -1407,7 +1458,7 @@ public class TransformerIdentityImpl extends Transformer
 
   /** The error listener for TrAX errors and warnings. */
   private ErrorListener m_errorListener =
-    new org.apache.xml.utils.DefaultErrorHandler();
+    new org.apache.xml.utils.DefaultErrorHandler(false);
 
   /**
    * The URIResolver, which is unused, but must be returned to fullfill the
@@ -1421,4 +1472,9 @@ public class TransformerIdentityImpl extends Transformer
   /** Flag to set if we've found the first element, so we can tell if we have 
    *  to check to see if we should create an HTML serializer.      */
   boolean m_foundFirstElement;
+  
+  /**
+   * State of the secure processing feature.
+   */
+  private boolean m_isSecureProcessing = false;
 }
