@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: FunctionCall.java,v 1.38 2004/02/23 17:29:35 aruny Exp $
+ * $Id: FunctionCall.java,v 1.41 2005/06/30 21:04:16 zongaro Exp $
  */
 
 package org.apache.xalan.xsltc.compiler;
@@ -35,15 +35,9 @@ import org.apache.bcel.generic.INVOKEVIRTUAL;
 import org.apache.bcel.generic.InstructionConstants;
 import org.apache.bcel.generic.InstructionList;
 import org.apache.bcel.generic.InvokeInstruction;
+import org.apache.bcel.generic.LocalVariableGen;
 import org.apache.bcel.generic.NEW;
 import org.apache.bcel.generic.PUSH;
-import org.apache.bcel.generic.LocalVariableGen;
-import org.apache.bcel.generic.ASTORE;
-import org.apache.bcel.generic.ANEWARRAY;
-import org.apache.bcel.generic.ALOAD;
-import org.apache.bcel.generic.ACONST_NULL;
-
-
 import org.apache.xalan.xsltc.compiler.util.BooleanType;
 import org.apache.xalan.xsltc.compiler.util.ClassGenerator;
 import org.apache.xalan.xsltc.compiler.util.ErrorMsg;
@@ -133,9 +127,6 @@ class FunctionCall extends Expression {
     // If the java method is static
     private boolean 	  _isStatic = false;
 
-    // If unable to resolve statically then resolve dynamically
-    private boolean  resolveDynamic = false;
-    
     // Legal conversions between internal and Java types.
     private static final MultiHashtable _internal2Java = new MultiHashtable();
 
@@ -494,14 +485,10 @@ class FunctionCall extends Expression {
 	}
 
 	if (_type != null) {
-           return _type;
+	    return _type;
 	}
-        else{
-          resolveDynamic = true;
-        }  
 
-        return  _type = (_clazz != null) ? Type.newObjectType(_clazz)
-                    : Type.newObjectType(_className);
+	throw new TypeCheckError(ErrorMsg.ARGUMENT_CONVERSION_ERR, getMethodSignature(argsType));
     }
 
 
@@ -540,11 +527,6 @@ class FunctionCall extends Expression {
 	  	    && _clazz != null
 	  	    && _clazz.isAssignableFrom(((ObjectType)firstArgType).getJavaClass()))
 	  	    hasThisArgument = true;
-                else if(firstArgType instanceof ReferenceType){
-                    resolveDynamic = true;
-              	    typeCheckArgs(stable);
-                    return Type.String;
-                }
 	  	
 	  	if (hasThisArgument) {
 	  	    _thisArgument = (Expression) _arguments.elementAt(0);
@@ -718,6 +700,7 @@ class FunctionCall extends Expression {
 	final int n = argumentCount();
 	final ConstantPoolGen cpg = classGen.getConstantPool();
 	final InstructionList il = methodGen.getInstructionList();
+	final boolean isSecureProcessing = classGen.getParser().getXSLTC().isSecureProcessing();
 	int index;
 
 	// Translate calls to methods in the BasisLibrary
@@ -761,21 +744,45 @@ class FunctionCall extends Expression {
 	    il.append(new INVOKESTATIC(index));
 	}
 	else if (_isExtConstructor) {
+	    if (isSecureProcessing)
+	        translateUnallowedExtension(cpg, il);
+	    
 	    final String clazz = 
 		_chosenConstructor.getDeclaringClass().getName();
 	    Class[] paramTypes = _chosenConstructor.getParameterTypes();
-	    
-	    il.append(new NEW(cpg.addClass(_className)));
-	    il.append(InstructionConstants.DUP);
+            LocalVariableGen[] paramTemp = new LocalVariableGen[n];
+
+            // Backwards branches are prohibited if an uninitialized object is
+            // on the stack by section 4.9.4 of the JVM Specification, 2nd Ed.
+            // We don't know whether this code might contain backwards branches
+            // so we mustn't create the new object until after we've created
+            // the suspect arguments to its constructor.  Instead we calculate
+            // the values of the arguments to the constructor first, store them
+            // in temporary variables, create the object and reload the
+            // arguments from the temporaries to avoid the problem.
 
 	    for (int i = 0; i < n; i++) {
 		final Expression exp = argument(i);
+                Type expType = exp.getType();
 		exp.translate(classGen, methodGen);
 		// Convert the argument to its Java type
 		exp.startIterator(classGen, methodGen);
-		exp.getType().translateTo(classGen, methodGen, paramTypes[i]);
+		expType.translateTo(classGen, methodGen, paramTypes[i]);
+                paramTemp[i] =
+                    methodGen.addLocalVariable("function_call_tmp"+i,
+                                               expType.toJCType(),
+                                               il.getEnd(), null);
+                il.append(expType.STORE(paramTemp[i].getIndex()));
 	    }
 
+	    il.append(new NEW(cpg.addClass(_className)));
+	    il.append(InstructionConstants.DUP);
+
+            for (int i = 0; i < n; i++) {
+                final Expression arg = argument(i);
+                il.append(arg.getType().LOAD(paramTemp[i].getIndex()));
+            }
+
 	    final StringBuffer buffer = new StringBuffer();
 	    buffer.append('(');
 	    for (int i = 0; i < paramTypes.length; i++) {
@@ -794,58 +801,11 @@ class FunctionCall extends Expression {
 				_chosenConstructor.getDeclaringClass());
 	    
 	}
-	else if(resolveDynamic) {
-
-          final LocalVariableGen _local = methodGen.addLocalVariable2("objects",
-              org.apache.bcel.generic.Type.OBJECT, il.getEnd());
-
-          //Create the Object[].
-          il.append(new PUSH(cpg, n + 1));
-          il.append(new ANEWARRAY(cpg.addClass(OBJECT_CLASS)));
-
-          // Push "this" if it is an instance method
-          il.append(DUP);
-          if (_thisArgument != null) {
-            il.append(new PUSH(cpg, 0));
-            _thisArgument.translate(classGen, methodGen);
-          }
-          //else add null to the Object array
-          else {
-            il.append(new PUSH(cpg, 0));
-            il.append(ACONST_NULL);
-          }
-          il.append(AASTORE);
-
-          //Add the parameters to Object[]
-          for (int i = 0; i < n; i++) {
-            il.append(DUP);
-            il.append(new PUSH(cpg, i + 1));
-            Expression exp = argument(i);
-            exp.translate(classGen, methodGen);
-            exp.startIterator(classGen, methodGen);
-            exp.getType().translateTo(classGen, methodGen, Type.Reference);
-            il.append(AASTORE);
-          }
-          il.append(new ASTORE(_local.getIndex()));
-
-          //call invoke parameter
-          String methodName = null;
-          if (_chosenMethod != null)
-            methodName = _chosenMethod.getName();
-          else
-            methodName = _fname.getLocalPart();
-
-          il.append(new PUSH(cpg, _className));
-          il.append(new PUSH(cpg, methodName));
-          il.append(new ALOAD(_local.getIndex()));
-
-          index = cpg.addMethodref(CALL_FUNCTION_CLASS, INVOKE_METHOD,
-              "(Ljava/lang/String;Ljava/lang/String;[Ljava/lang/Object;)Ljava/lang/String;");
-          il.append(new INVOKESTATIC(index));
-
-        }
 	// Invoke function calls that are handled in separate classes
 	else {
+	    if (isSecureProcessing)
+	        translateUnallowedExtension(cpg, il);
+	    
 	    final String clazz = _chosenMethod.getDeclaringClass().getName();
 	    Class[] paramTypes = _chosenMethod.getParameterTypes();
 
@@ -1114,4 +1074,16 @@ class FunctionCall extends Expression {
         return buff.toString();
     }
  	 
+    /**
+     * Translate code to call the BasisLibrary.unallowed_extensionF(String)
+     * method.
+     */
+    private void translateUnallowedExtension(ConstantPoolGen cpg,
+                                             InstructionList il) {
+	int index = cpg.addMethodref(BASIS_LIBRARY_CLASS,
+				     "unallowed_extension_functionF",
+				     "(Ljava/lang/String;)V");
+	il.append(new PUSH(cpg, _fname.toString()));
+	il.append(new INVOKESTATIC(index));   
+    } 	 
 }
