@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: Mode.java,v 1.31 2004/02/16 22:24:29 minchau Exp $
+ * $Id: Mode.java,v 1.36 2005/02/17 19:08:02 ytalwar Exp $
  */
 
 package org.apache.xalan.xsltc.compiler;
@@ -24,6 +24,7 @@ import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.Vector;
 
+import org.apache.bcel.generic.Instruction;
 import org.apache.bcel.generic.BranchHandle;
 import org.apache.bcel.generic.ConstantPoolGen;
 import org.apache.bcel.generic.DUP;
@@ -44,7 +45,7 @@ import org.apache.xalan.xsltc.compiler.util.ClassGenerator;
 import org.apache.xalan.xsltc.compiler.util.MethodGenerator;
 import org.apache.xalan.xsltc.compiler.util.NamedMethodGenerator;
 import org.apache.xalan.xsltc.compiler.util.Util;
-import org.apache.xalan.xsltc.dom.Axis;
+import org.apache.xml.dtm.Axis;
 import org.apache.xml.dtm.DTM;
 
 /**
@@ -119,14 +120,7 @@ final class Mode implements Constants {
      */
     private TestSeq[] _testSeq;
 
-    /**
-     * A mapping between patterns and instruction lists used by 
-     * test sequences to avoid compiling the same pattern multiple 
-     * times. Note that patterns whose kernels are "*", "node()" 
-     * and "@*" can between shared by test sequences.
-     */
-    private Hashtable _preCompiled = new Hashtable();
-
+    
     /**
      * A mapping between templates and test sequences.
      */
@@ -203,23 +197,6 @@ final class Mode implements Constants {
 	return _methodName + '_' + max;
     }
 
-    /**
-     * Add a pre-compiled pattern to this mode. 
-     */
-    public void addInstructionList(Pattern pattern, 
-	InstructionList ilist) 
-    {
-	_preCompiled.put(pattern, ilist);
-    }
-
-    /**
-     * Get the instruction list for a pre-compiled pattern. Used by 
-     * test sequences to avoid compiling patterns more than once.
-     */
-    public InstructionList getInstructionList(Pattern pattern) {
-	return (InstructionList) _preCompiled.get(pattern);
-    }
-
     /**
      * Shortcut to get the class compiled for this mode (will be inlined).
      */
@@ -1125,15 +1102,17 @@ for (int i = 0; i < _templates.size(); i++) {
 
 	// Create the applyTemplates() method
 	final org.apache.bcel.generic.Type[] argTypes =
-	    new org.apache.bcel.generic.Type[3];
+	    new org.apache.bcel.generic.Type[4];
 	argTypes[0] = Util.getJCRefType(DOM_INTF_SIG);
 	argTypes[1] = Util.getJCRefType(NODE_ITERATOR_SIG);
 	argTypes[2] = Util.getJCRefType(TRANSLET_OUTPUT_SIG);
+	argTypes[3] = org.apache.bcel.generic.Type.INT;
 
-	final String[] argNames = new String[3];
+	final String[] argNames = new String[4];
 	argNames[0] = DOCUMENT_PNAME;
 	argNames[1] = ITERATOR_PNAME;
 	argNames[2] = TRANSLET_OUTPUT_PNAME;
+	argNames[3] = NODE_PNAME;
 
 	final InstructionList mainIL = new InstructionList();
 	final MethodGenerator methodGen =
@@ -1151,6 +1130,9 @@ for (int i = 0; i < _templates.size(); i++) {
 					      mainIL.getEnd());
 	_currentIndex = current.getIndex();
 
+    mainIL.append(new ILOAD(methodGen.getLocalIndex(NODE_PNAME)));
+    mainIL.append(new ISTORE(_currentIndex));
+    
 	// Create the "body" instruction list that will eventually hold the
 	// code for the entire method (other ILs will be appended).
 	final InstructionList body = new InstructionList();
@@ -1159,16 +1141,7 @@ for (int i = 0; i < _templates.size(); i++) {
 	// Create an instruction list that contains the default next-node
 	// iteration
 	final InstructionList ilLoop = new InstructionList();
-	ilLoop.append(methodGen.loadIterator());
-	ilLoop.append(methodGen.nextNode());
-	ilLoop.append(DUP);
-	ilLoop.append(new ISTORE(_currentIndex));
-
-	// The body of this code can get very large - large than can be handled
-	// by a single IFNE(body.getStart()) instruction - need workaround:
-        final BranchHandle ifeq = ilLoop.append(new IFLT(null));
-	final BranchHandle loop = ilLoop.append(new GOTO_W(null));
-	ifeq.setTarget(ilLoop.append(RETURN)); // applyTemplates() ends here!
+    ilLoop.append(RETURN);
 	final InstructionHandle ihLoop = ilLoop.getStart();
 
 	// Compile default handling of elements (traverse children)
@@ -1215,12 +1188,8 @@ for (int i = 0; i < _templates.size(); i++) {
 	// Do tests for id() and key() patterns first
 	InstructionList ilKey = null;
 	if (_idxTestSeq != null) {
-	    loop.setTarget(_idxTestSeq.compile(classGen, methodGen, body.getStart()));
 	    ilKey = _idxTestSeq.getInstructionList();
 	}
-	else {
-	    loop.setTarget(body.getStart());
-	}
 
 	// If there is a match on node() we need to replace ihElem
 	// and ihText if the priority of node() is higher
@@ -1411,7 +1380,6 @@ for (int i = 0; i < _templates.size(); i++) {
 	body.append(ilText);
 
 	// putting together constituent instruction lists
-	mainIL.append(new GOTO_W(ihLoop));
 	mainIL.append(body);
 	// fall through to ilLoop
 	mainIL.append(ilLoop);
@@ -1429,20 +1397,20 @@ for (int i = 0; i < _templates.size(); i++) {
     }
 
     /**
-     * Peephole optimization: Remove sequences of [ALOAD, POP].
-     */
+      * Peephole optimization.
+      */
     private void peepHoleOptimization(MethodGenerator methodGen) {
-    InstructionList il = methodGen.getInstructionList();
-    InstructionFinder find = new InstructionFinder(il);
+        InstructionList il = methodGen.getInstructionList();
+        InstructionFinder find = new InstructionFinder(il);
 	InstructionHandle ih;
 	String pattern;
 
-	// Remove seqences of ALOAD, POP (GTM)
-	pattern = "`ALOAD'`POP'`Instruction'";
-	for(Iterator iter=find.search(pattern); iter.hasNext();){
-	    InstructionHandle[] match = (InstructionHandle[])iter.next();
+	// LoadInstruction, POP => (removed)
+	pattern = "LoadInstruction POP";
+	for (Iterator iter = find.search(pattern); iter.hasNext();) {
+	    InstructionHandle[] match = (InstructionHandle[]) iter.next();
 	    try {
-		if ((!match[0].hasTargeters()) && (!match[1].hasTargeters())) {
+		if (!match[0].hasTargeters() && !match[1].hasTargeters()) {
                     il.delete(match[0], match[1]);
                 }
 	    }
@@ -1450,45 +1418,73 @@ for (int i = 0; i < _templates.size(); i++) {
                 // TODO: move target down into the list
             }
 	}
-	// Replace sequences of ILOAD_?, ALOAD_?, SWAP with ALOAD_?, ILOAD_?
-	pattern = "`ILOAD'`ALOAD'`SWAP'`Instruction'";
-	for(Iterator iter=find.search(pattern); iter.hasNext();){
-            InstructionHandle[] match = (InstructionHandle[])iter.next();
-            try {
-                org.apache.bcel.generic.Instruction iload;
-                org.apache.bcel.generic.Instruction aload;
-                if ((!match[0].hasTargeters()) &&
-                    (!match[1].hasTargeters()) &&
-                    (!match[2].hasTargeters())) {
-                    iload = match[0].getInstruction();
-                    aload = match[1].getInstruction();
-                    il.insert(match[0], aload);
-                    il.insert(match[0], iload);
-                    il.delete(match[0], match[2]);
+        
+	// ILOAD_N, ILOAD_N, SWAP, ISTORE_N => ILOAD_N
+	pattern = "ILOAD ILOAD SWAP ISTORE";
+	for (Iterator iter = find.search(pattern); iter.hasNext();) {
+            InstructionHandle[] match = (InstructionHandle[]) iter.next();
+            try {              
+                org.apache.bcel.generic.ILOAD iload1 = 
+                    (org.apache.bcel.generic.ILOAD) match[0].getInstruction();
+                org.apache.bcel.generic.ILOAD iload2 = 
+                    (org.apache.bcel.generic.ILOAD) match[1].getInstruction();
+                org.apache.bcel.generic.ISTORE istore = 
+                    (org.apache.bcel.generic.ISTORE) match[3].getInstruction();
+                
+                if (!match[1].hasTargeters() &&
+                    !match[2].hasTargeters() &&
+                    !match[3].hasTargeters() &&
+                    iload1.getIndex() == iload2.getIndex() &&
+                    iload2.getIndex() == istore.getIndex())
+                {
+                    il.delete(match[1], match[3]);
                 }
             }
             catch (TargetLostException e) {
                 // TODO: move target down into the list
             }
         }
-        
-        // Replace sequences of ALOAD_1, ALOAD_1 with ALOAD_1, DUP 
-	pattern = "`ALOAD_1'`ALOAD_1'`Instruction'";
-        for(Iterator iter=find.search(pattern); iter.hasNext();){
+
+        // LoadInstruction_N, LoadInstruction_M, SWAP => LoadInstruction_M, LoadInstruction_N
+	pattern = "LoadInstruction LoadInstruction SWAP";
+	for (Iterator iter = find.search(pattern); iter.hasNext();) {
             InstructionHandle[] match = (InstructionHandle[])iter.next();
             try {
-	        org.apache.bcel.generic.Instruction iload;
-                org.apache.bcel.generic.Instruction aload;
-                if ((!match[0].hasTargeters()) && (!match[1].hasTargeters())) {
-                    il.insert(match[1], new DUP());
-                    il.delete(match[1]);
+                if (!match[0].hasTargeters() &&
+                    !match[1].hasTargeters() &&
+                    !match[2].hasTargeters()) 
+                {
+                    Instruction load_m = match[1].getInstruction();
+                    il.insert(match[0], load_m);
+                    il.delete(match[1], match[2]);
+                }
+            }
+            catch (TargetLostException e) {
+                // TODO: move target down into the list
+            }
+        }
+                
+        // ALOAD_N ALOAD_N => ALOAD_N DUP
+	pattern = "ALOAD ALOAD";
+        for (Iterator iter = find.search(pattern); iter.hasNext();) {
+            InstructionHandle[] match = (InstructionHandle[])iter.next();
+            try {
+                if (!match[1].hasTargeters()) {
+                    org.apache.bcel.generic.ALOAD aload1 = 
+                        (org.apache.bcel.generic.ALOAD) match[0].getInstruction();
+                    org.apache.bcel.generic.ALOAD aload2 = 
+                        (org.apache.bcel.generic.ALOAD) match[1].getInstruction();
+                    
+                    if (aload1.getIndex() == aload2.getIndex()) {
+                        il.insert(match[1], new DUP());
+                        il.delete(match[1]);
+                    }
                 }
             }
             catch (TargetLostException e) {
                 // TODO: move target down into the list
             }
         }
-        
     }
 
     public InstructionHandle getTemplateInstructionHandle(Template template) {
