@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: Stylesheet.java,v 1.59 2004/02/16 22:24:29 minchau Exp $
+ * $Id: Stylesheet.java,v 1.61 2004/05/26 21:33:08 jycli Exp $
  */
 
 package org.apache.xalan.xsltc.compiler;
@@ -59,11 +59,6 @@ import org.apache.xalan.xsltc.compiler.util.Util;
 import org.apache.xalan.xsltc.runtime.AbstractTranslet;
 import org.apache.xml.dtm.DTM;
 
-/**
- * @author Jacek Ambroziak
- * @author Santiago Pericas-Geertsen
- * @author Morten Jorgensen
- */
 public final class Stylesheet extends SyntaxTreeNode {
 
     /**
@@ -153,7 +148,13 @@ public final class Stylesheet extends SyntaxTreeNode {
      * Import precendence for this stylesheet.
      */
     private int _importPrecedence = 1;
-    
+
+    /**
+     * Minimum precendence of any descendant stylesheet by inclusion or
+     * importation.
+     */
+    private int _minimumDescendantPrecedence = -1;
+
     /**
      * Mapping between key names and Key objects (needed by Key/IdPattern).
      */
@@ -345,6 +346,35 @@ public final class Stylesheet extends SyntaxTreeNode {
 	return _importPrecedence;
     }
 
+    /**
+     * Get the minimum of the precedence of this stylesheet, any stylesheet
+     * imported by this stylesheet and any include/import descendant of this
+     * stylesheet.
+     */
+    public int getMinimumDescendantPrecedence() {
+        if (_minimumDescendantPrecedence == -1) {
+            // Start with precedence of current stylesheet as a basis.
+            int min = getImportPrecedence();
+
+            // Recursively examine all imported/included stylesheets.
+            final int inclImpCount = (_includedStylesheets != null)
+                                          ? _includedStylesheets.size()
+                                          : 0;
+
+            for (int i = 0; i < inclImpCount; i++) {
+                int prec = ((Stylesheet)_includedStylesheets.elementAt(i))
+                                              .getMinimumDescendantPrecedence();
+
+                if (prec < min) {
+                    min = prec;
+                }
+            }
+
+            _minimumDescendantPrecedence = min;
+        }
+        return _minimumDescendantPrecedence;
+    }
+
     public boolean checkForLoop(String systemId) {
 	// Return true if this stylesheet includes/imports itself
 	if (_systemId != null && _systemId.equals(systemId)) {
@@ -1291,7 +1321,7 @@ public final class Stylesheet extends SyntaxTreeNode {
      * Peephole optimization: Remove sequences of [ALOAD, POP].
      */
     private void peepHoleOptimization(MethodGenerator methodGen) {
-	final String pattern = "`ALOAD'`POP'`Instruction'";
+	final String pattern = "`aload'`pop'`instruction'";
 	final InstructionList il = methodGen.getInstructionList();
 	final InstructionFinder find = new InstructionFinder(il);
 	for(Iterator iter=find.search(pattern); iter.hasNext(); ) {
