@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: SyntaxTreeNode.java,v 1.29 2004/02/16 22:25:10 minchau Exp $
+ * $Id: SyntaxTreeNode.java,v 1.32 2004/12/15 17:35:39 jycli Exp $
  */
 
 package org.apache.xalan.xsltc.compiler;
@@ -43,10 +43,12 @@ import org.apache.xalan.xsltc.compiler.util.MethodGenerator;
 import org.apache.xalan.xsltc.compiler.util.Type;
 import org.apache.xalan.xsltc.compiler.util.TypeCheckError;
 import org.apache.xalan.xsltc.DOM;
+import org.apache.xalan.xsltc.runtime.AttributeList;
 
 import org.xml.sax.Attributes;
 
 
+
 /**
  * @author Jacek Ambroziak
  * @author Santiago Pericas-Geertsen
@@ -69,11 +71,11 @@ public abstract class SyntaxTreeNode implements Constants {
     // Element description data
     protected QName _qname;                    // The element QName
     private int _line;                         // Source file line number
-    protected Attributes _attributes = null;   // Attributes of this element
+    protected AttributeList _attributes = null;   // Attributes of this element
     private   Hashtable _prefixMapping = null; // Namespace declarations
 
     // Sentinel - used to denote unrecognised syntaxt tree nodes.
-    protected static final SyntaxTreeNode Dummy = new AbsolutePathPattern(null);
+    static final SyntaxTreeNode Dummy = new AbsolutePathPattern(null);
 
     // These two are used for indenting nodes in the AST (debug output)
     protected static final int IndentIncrement = 4;
@@ -118,11 +120,15 @@ public abstract class SyntaxTreeNode implements Constants {
     }
 
     /**
-     * Get the source file line number for this element
+     * Get the source file line number for this element. If unavailable, lookup
+     * in ancestors.
+     *
      * @return The source file line number.
      */
     public final int getLineNumber() {
-	return _line;
+        if (_line > 0) return _line;
+        SyntaxTreeNode parent = getParent();
+        return (parent != null) ? parent.getLineNumber() : 0;
     }
 
     /**
@@ -156,7 +162,7 @@ public abstract class SyntaxTreeNode implements Constants {
      * @param attributes Attributes for the element. Must be passed in as an
      *                   implementation of org.xml.sax.Attributes.
      */
-    protected void setAttributes(Attributes attributes) {
+    protected void setAttributes(AttributeList attributes) {
 	_attributes = attributes;
     }
 
@@ -173,10 +179,18 @@ public abstract class SyntaxTreeNode implements Constants {
 	return (value == null || value.equals(EMPTYSTRING)) ? 
 	    EMPTYSTRING : value;
     }
+    
+    protected String getAttribute(String prefix, String localName) {
+        return getAttribute(prefix + ':' + localName);
+    }
 
     protected boolean hasAttribute(String qname) {
 	return (_attributes != null && _attributes.getValue(qname) != null);
     }
+    
+    protected void addAttribute(String qname, String value) {
+        _attributes.add(qname, value);
+    }
 
     /**
      * Returns a list of all attributes declared for the element represented by
