@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: MultiDOM.java,v 1.31 2004/02/16 22:54:59 minchau Exp $
+ * $Id: MultiDOM.java,v 1.32 2004/12/15 17:35:44 jycli Exp $
  */
 
 package org.apache.xalan.xsltc.dom;
@@ -26,6 +26,7 @@ import org.apache.xalan.xsltc.TransletException;
 import org.apache.xalan.xsltc.runtime.BasisLibrary;
 import org.apache.xalan.xsltc.runtime.Hashtable;
 import org.apache.xml.dtm.DTM;
+import org.apache.xml.dtm.Axis;
 import org.apache.xml.dtm.DTMAxisIterator;
 import org.apache.xml.dtm.DTMManager;
 import org.apache.xml.dtm.ref.DTMAxisIteratorBase;
@@ -129,7 +130,7 @@ public final class MultiDOM implements DOM {
         }
     
         public boolean isReverse() {
-	    return Axis.isReverse[_axis];
+            return Axis.isReverse(_axis);
         }
     
         public void setMark() {
