@@ -14,11 +14,12 @@
  * limitations under the License.
  */
 /*
- * $Id: AbstractTranslet.java,v 1.52 2004/02/16 22:55:55 minchau Exp $
+ * $Id: AbstractTranslet.java,v 1.58 2005/07/11 13:18:28 zongaro Exp $
  */
 
 package org.apache.xalan.xsltc.runtime;
 
+import java.io.File;
 import java.io.FileWriter;
 import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
@@ -26,6 +27,10 @@ import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.Vector;
 import javax.xml.transform.Templates;
+import javax.xml.parsers.DocumentBuilderFactory;
+import org.w3c.dom.Document;
+import org.w3c.dom.DOMImplementation;
+import javax.xml.parsers.ParserConfigurationException;
 
 import org.apache.xml.dtm.DTM;
 
@@ -61,6 +66,7 @@ public abstract class AbstractTranslet implements Translet {
     public boolean _indent = false;
     public String  _mediaType = null;
     public Vector _cdata = null;
+    public int _indentamount = -1;
 
     public static final int FIRST_TRANSLET_VERSION = 100;
     public static final int VER_SPLIT_NAMES_ARRAY = 101;
@@ -420,9 +426,9 @@ public abstract class AbstractTranslet implements Translet {
 
     /**
      * Adds a value to a key/id index
-     *   @name is the name of the index (the key or ##id)
-     *   @node is the node id of the node to insert
-     *   @value is the value that will look up the node in the given index
+     *   @param name is the name of the index (the key or ##id)
+     *   @param node is the node id of the node to insert
+     *   @param value is the value that will look up the node in the given index
      */
     public void buildKeyIndex(String name, int node, Object value) {
 	if (_keyIndexes == null) _keyIndexes = new Hashtable();
@@ -436,8 +442,8 @@ public abstract class AbstractTranslet implements Translet {
 
     /**
      * Create an empty KeyIndex in the DOM case
-     *   @name is the name of the index (the key or ##id)
-     *   @node is the DOM
+     *   @param name is the name of the index (the key or ##id)
+     *   @param dom is the DOM
      */
     public void buildKeyIndex(String name, DOM dom) {
 	if (_keyIndexes == null) _keyIndexes = new Hashtable();
@@ -528,6 +534,12 @@ public abstract class AbstractTranslet implements Translet {
 	    final TransletOutputHandlerFactory factory 
 		= TransletOutputHandlerFactory.newInstance();
 
+            String dirStr = new File(filename).getParent();
+            if ((null != dirStr) && (dirStr.length() > 0)) {
+               File dir = new File(dirStr);
+               dir.mkdirs();
+            }
+
 	    factory.setEncoding(_encoding);
 	    factory.setOutputMethod(_method);
 	    factory.setWriter(new FileWriter(filename, append));
@@ -577,7 +589,11 @@ public abstract class AbstractTranslet implements Translet {
      */
     public final void transform(DOM document, SerializationHandler handler) 
 	throws TransletException {
-	transform(document, document.getIterator(), handler);
+        try {
+            transform(document, document.getIterator(), handler);
+        } finally {
+            _keyIndexes = null;
+        }
     }
 	
     /**
@@ -635,6 +651,7 @@ public abstract class AbstractTranslet implements Translet {
 		    handler.setVersion(_version);
 		}
 		handler.setIndent(_indent);
+		handler.setIndentAmount(_indentamount);
 		if (_doctypeSystem != null) {
 		    handler.setDoctype(_doctypeSystem, _doctypePublic);
 		}
@@ -707,4 +724,19 @@ public abstract class AbstractTranslet implements Translet {
     public void setTemplates(Templates templates) {
     	_templates = templates;
     }    
+    
+    /************************************************************************
+     * DOMImplementation caching for basis library
+     ************************************************************************/
+    protected DOMImplementation _domImplementation = null;
+    
+    public Document newDocument(String uri, String qname) 
+        throws ParserConfigurationException 
+    {
+        if (_domImplementation == null) {
+            _domImplementation = DocumentBuilderFactory.newInstance()
+                .newDocumentBuilder().getDOMImplementation();
+        }
+        return _domImplementation.createDocument(uri, qname, null);
+    }
 }
