@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: BasisLibrary.java,v 1.76 2004/02/27 01:59:31 zongaro Exp $
+ * $Id: BasisLibrary.java,v 1.83 2005/07/13 04:43:27 minchau Exp $
  */
 
 package org.apache.xalan.xsltc.runtime;
@@ -26,14 +26,12 @@ import java.text.NumberFormat;
 import java.util.Locale;
 import java.util.ResourceBundle;
 
-import javax.xml.parsers.DocumentBuilder;
-import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.transform.dom.DOMSource;
 
 import org.apache.xalan.xsltc.DOM;
 import org.apache.xalan.xsltc.Translet;
 import org.apache.xalan.xsltc.dom.AbsoluteIterator;
-import org.apache.xalan.xsltc.dom.Axis;
+import org.apache.xml.dtm.Axis;
 import org.apache.xalan.xsltc.dom.DOMAdapter;
 import org.apache.xalan.xsltc.dom.MultiDOM;
 import org.apache.xalan.xsltc.dom.SingletonIterator;
@@ -48,13 +46,13 @@ import org.w3c.dom.NodeList;
 import org.xml.sax.SAXException;
 import org.apache.xml.serializer.NamespaceMappings;
 import org.apache.xml.serializer.SerializationHandler;
-import org.apache.xml.utils.XMLChar;
+import org.apache.xml.utils.XML11Char;
 
 /**
  * Standard XSLT functions. All standard functions expect the current node 
  * and the DOM as their last two arguments.
  */
-public final class BasisLibrary implements Operators {
+public final class BasisLibrary {
 
     private final static String EMPTYSTRING = "";
 
@@ -231,7 +229,7 @@ public final class BasisLibrary implements Operators {
 	}
 	else {
 	    final String className = obj.getClass().getName();
-	    runTimeError(INVALID_ARGUMENT_ERR, className, "number()");
+	    runTimeError(INVALID_ARGUMENT_ERR, className, "boolean()");
 	}
 	return false;
     }
@@ -401,6 +399,22 @@ public final class BasisLibrary implements Operators {
 	runTimeError(EXTERNAL_FUNC_ERR, name);
     }
 
+    /**
+     * Utility function to throw a runtime error on the use of an extension 
+     * function when the secure processing feature is set to true.
+     */
+    public static void unallowed_extension_functionF(String name) {
+        runTimeError(UNALLOWED_EXTENSION_FUNCTION_ERR, name);
+    }
+
+    /**
+     * Utility function to throw a runtime error on the use of an extension 
+     * element when the secure processing feature is set to true.
+     */
+    public static void unallowed_extension_elementF(String name) {
+        runTimeError(UNALLOWED_EXTENSION_ELEMENT_ERR, name);
+    }
+
     /**
      * Utility function to throw a runtime error for an unsupported element.
      * 
@@ -498,22 +512,22 @@ public final class BasisLibrary implements Operators {
     private static boolean compareStrings(String lstring, String rstring,
 					  int op, DOM dom) {
 	switch (op) {
-	case EQ:
+    case Operators.EQ:
 	    return lstring.equals(rstring);
 
-	case NE:
+    case Operators.NE:
 	    return !lstring.equals(rstring);
 
-	case GT:
+    case Operators.GT:
 	    return numberF(lstring, dom) > numberF(rstring, dom);
 
-	case LT:
+    case Operators.LT:
 	    return numberF(lstring, dom) < numberF(rstring, dom);
 
-	case GE:
+    case Operators.GE:
 	    return numberF(lstring, dom) >= numberF(rstring, dom);
-
-	case LE:
+        
+    case Operators.LE:
 	    return numberF(lstring, dom) <= numberF(rstring, dom);
 
 	default:
@@ -538,9 +552,9 @@ public final class BasisLibrary implements Operators {
 	    while ((rnode = right.next()) != DTMAxisIterator.END) {
                 // String value must be the same if both nodes are the same
                 if (lnode == rnode) {
-                    if (op == EQ) {
+                    if (op == Operators.EQ) {
                         return true;
-                    } else if (op == NE) {
+                    } else if (op == Operators.NE) {
                         continue;
                     }
                 }
@@ -561,7 +575,7 @@ public final class BasisLibrary implements Operators {
 	String value;
 
 	switch(op) {
-	case EQ:
+    case Operators.EQ:
             rnode = iterator.next();
             if (rnode != DTMAxisIterator.END) {
 	        value = dom.getStringValueX(node);
@@ -573,7 +587,7 @@ public final class BasisLibrary implements Operators {
 	        } while ((rnode = iterator.next()) != DTMAxisIterator.END);
             }
 	    break;
-	case NE:
+    case Operators.NE:
             rnode = iterator.next();
             if (rnode != DTMAxisIterator.END) {
 	        value = dom.getStringValueX(node);
@@ -585,13 +599,13 @@ public final class BasisLibrary implements Operators {
 	        } while ((rnode = iterator.next()) != DTMAxisIterator.END);
             }
 	    break;
-	case LT:
+    case Operators.LT:
 	    // Assume we're comparing document order here
 	    while ((rnode = iterator.next()) != DTMAxisIterator.END) {
 		if (rnode > node) return true;
 	    }
 	    break;
-	case GT:
+    case Operators.GT:
 	    // Assume we're comparing document order here
 	    while ((rnode = iterator.next()) != DTMAxisIterator.END) {
 		if (rnode < node) return true;
@@ -610,42 +624,42 @@ public final class BasisLibrary implements Operators {
 	//left.reset();
 
 	switch (op) {
-	case EQ:
+    case Operators.EQ:
 	    while ((node = left.next()) != DTMAxisIterator.END) {
 		if (numberF(dom.getStringValueX(node), dom) == rnumber)
 		    return true;
 	    }
 	    break;
 
-	case NE:
+    case Operators.NE:
 	    while ((node = left.next()) != DTMAxisIterator.END) {
 		if (numberF(dom.getStringValueX(node), dom) != rnumber)
 		    return true;
 	    }
 	    break;
 
-	case GT:
+    case Operators.GT:
 	    while ((node = left.next()) != DTMAxisIterator.END) {
 		if (numberF(dom.getStringValueX(node), dom) > rnumber)
 		    return true;
 	    }
 	    break;
 
-	case LT:
+    case Operators.LT:
 	    while ((node = left.next()) != DTMAxisIterator.END) {
 		if (numberF(dom.getStringValueX(node), dom) < rnumber)
 		    return true;
 	    }
 	    break;
 
-	case GE:
+    case Operators.GE:
 	    while ((node = left.next()) != DTMAxisIterator.END) {
 		if (numberF(dom.getStringValueX(node), dom) >= rnumber)
 		    return true;
 	    }
 	    break;
 
-	case LE:
+    case Operators.LE:
 	    while ((node = left.next()) != DTMAxisIterator.END) {
 		if (numberF(dom.getStringValueX(node), dom) <= rnumber)
 		    return true;
@@ -681,7 +695,7 @@ public final class BasisLibrary implements Operators {
 	boolean result = false;
 	boolean hasSimpleArgs = hasSimpleType(left) && hasSimpleType(right);
 
-	if (op != EQ && op != NE) {
+    if (op != Operators.EQ && op != Operators.NE) {
 	    // If node-boolean comparison -> convert node to boolean
 	    if (left instanceof Node || right instanceof Node) {
 		if (left instanceof Boolean) {
@@ -696,19 +710,19 @@ public final class BasisLibrary implements Operators {
 
 	    if (hasSimpleArgs) {
 		switch (op) {
-		case GT:
+        case Operators.GT:
 		    return numberF(left, dom) > numberF(right, dom);
 		    
-		case LT:
+        case Operators.LT:
 		    return numberF(left, dom) < numberF(right, dom);
 		    
-		case GE:
+        case Operators.GE:
 		    return numberF(left, dom) >= numberF(right, dom);
 		    
-		case LE:
+        case Operators.LE:
 		    return numberF(left, dom) <= numberF(right, dom);
 		    
-		default:
+        default:
 		    runTimeError(RUN_TIME_INTERNAL_ERR, "compare()");
 		}
 	    }
@@ -741,8 +755,9 @@ public final class BasisLibrary implements Operators {
 
 	    if (hasSimpleType(left) ||
 		left instanceof DOM && right instanceof DTMAxisIterator) {
-		// swap operands
+		// swap operands and operator
 		final Object temp = right; right = left; left = temp;
+                op = Operators.swapOp(op);
 	    }
 
 	    if (left instanceof DOM) {
@@ -1175,22 +1190,17 @@ public final class BasisLibrary implements Operators {
                                     	Translet translet, DOM dom) 
     {
 	// w3c NodeList -> w3c DOM
-	DocumentBuilderFactory dfac = DocumentBuilderFactory.newInstance();
-	DocumentBuilder docbldr = null;
+	Document doc = null;
 	try {
-	    docbldr = dfac.newDocumentBuilder();
-	} catch (javax.xml.parsers.ParserConfigurationException e) {
+	    doc = ((AbstractTranslet) translet).newDocument("", "__top__");
+	} 
+        catch (javax.xml.parsers.ParserConfigurationException e) {
 	    runTimeError(RUN_TIME_INTERNAL_ERR, e.getMessage());
             return null;
-
 	}
-	// create new w3c DOM
-	Document doc = docbldr.newDocument();	
-        org.w3c.dom.Node topElementNode = 
-            doc.appendChild(doc.createElementNS("", "__top__"));
-
+        
         // Copy all the nodes in the nodelist to be under the top element
-        copyNodes(nodeList, doc, topElementNode);
+        copyNodes(nodeList, doc, doc.getDocumentElement());
 
         // w3cDOM -> DTM -> DOMImpl
 	if (dom instanceof MultiDOM) {
@@ -1297,20 +1307,20 @@ public final class BasisLibrary implements Operators {
         
             if (firstOccur != lastOccur) {
                final String oriPrefix = name.substring(firstOccur+1, lastOccur); 
-                if (!XMLChar.isValidNCName(oriPrefix)) {
+                if (!XML11Char.isXML11ValidNCName(oriPrefix)) {
                     // even though the orignal prefix is ignored, it should still get checked for valid NCName
                     runTimeError(INVALID_QNAME_ERR,oriPrefix+":"+localName);
                 }
             }
             
             // prefix must be a valid NCName
-            if (!XMLChar.isValidNCName(newPrefix)) {
+            if (!XML11Char.isXML11ValidNCName(newPrefix)) {
                 runTimeError(INVALID_QNAME_ERR,newPrefix+":"+localName); 
             }  
         }
                 
         // local name must be a valid NCName and must not be XMLNS
-        if ((!XMLChar.isValidNCName(localName))||(localName.equals(Constants.XMLNS_PREFIX))) {
+        if ((!XML11Char.isXML11ValidNCName(localName))||(localName.equals(Constants.XMLNS_PREFIX))) {
             runTimeError(INVALID_QNAME_ERR,localName); 
         }
     }
@@ -1320,7 +1330,7 @@ public final class BasisLibrary implements Operators {
      * This method should only be invoked if the attribute value is an AVT
      */    
     public static void checkNCName(String name) {
-        if (!XMLChar.isValidNCName(name)) {
+        if (!XML11Char.isXML11ValidNCName(name)) {
             runTimeError(INVALID_NCNAME_ERR,name); 
         }  
     }        
@@ -1330,7 +1340,7 @@ public final class BasisLibrary implements Operators {
      * This method should only be invoked if the attribute value is an AVT
      */    
     public static void checkQName(String name) {
-        if (!XMLChar.isValidQName(name)) {
+        if (!XML11Char.isXML11ValidQName(name)) {
             runTimeError(INVALID_QNAME_ERR,name); 
         }  
     }
@@ -1447,9 +1457,11 @@ public final class BasisLibrary implements Operators {
                                            "UNKNOWN_TRANSLET_VERSION_ERR";
     public static final String INVALID_QNAME_ERR = "INVALID_QNAME_ERR";                                           
     public static final String INVALID_NCNAME_ERR = "INVALID_NCNAME_ERR";
+    public static final String UNALLOWED_EXTENSION_FUNCTION_ERR = "UNALLOWED_EXTENSION_FUNCTION_ERR";
+    public static final String UNALLOWED_EXTENSION_ELEMENT_ERR = "UNALLOWED_EXTENSION_ELEMENT_ERR";
 
     // All error messages are localized and are stored in resource bundles.
-    protected static ResourceBundle m_bundle;
+    private static ResourceBundle m_bundle;
     
     public final static String ERROR_MESSAGES_KEY = "error-messages";
 
