@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: TransformerImpl.java,v 1.80 2004/02/23 21:33:15 igorh Exp $
+ * $Id: TransformerImpl.java,v 1.86 2005/06/08 13:10:08 mkwan Exp $
  */
 
 package org.apache.xalan.xsltc.trax;
@@ -152,6 +152,11 @@ public final class TransformerImpl extends Transformer
      */
     private TransformerFactoryImpl _tfactory = null;
 
+    /**
+     * A reference to the output stream, if we create one in our code.
+     */
+    private OutputStream _ostream = null;
+
     /**
      * A reference to the XSLTCDTMManager which is used to build the DOM/DTM
      * for this transformer.
@@ -174,6 +179,11 @@ public final class TransformerImpl extends Transformer
      */
     private boolean _isIdentity = false;
 
+    /**
+     * State of the secure processing feature.
+     */
+    private boolean _isSecureProcessing = false;
+
     /**
      * A hashtable to store parameters for the identity transform. These
      * are not needed during the transformation, but we must keep track of 
@@ -228,6 +238,20 @@ public final class TransformerImpl extends Transformer
 	//_isIncremental = tfactory._incremental;
     }
 
+    /**
+     * Return the state of the secure processing feature.
+     */
+    public boolean isSecureProcessing() {
+        return _isSecureProcessing;
+    }
+    
+    /**
+     * Set the state of the secure processing feature.
+     */
+    public void setSecureProcessing(boolean flag) {
+        _isSecureProcessing = flag;
+    }
+
     /**
      * Returns the translet wrapped inside this Transformer or
      * null if this is the identity transform.
@@ -331,6 +355,7 @@ public final class TransformerImpl extends Transformer
             }
 	    else if (result instanceof DOMResult) {
 		_tohFactory.setNode(((DOMResult) result).getNode());
+		_tohFactory.setNextSibling(((DOMResult) result).getNextSibling());
 		_tohFactory.setOutputType(TransletOutputHandlerFactory.DOM);
 		return _tohFactory.getSerializationHandler();
             }
@@ -372,20 +397,20 @@ public final class TransformerImpl extends Transformer
 		if (systemId.startsWith("file:")) {
                     url = new URL(systemId);
 		    _tohFactory.setOutputStream(
-		        new FileOutputStream(url.getFile()));
+		        _ostream = new FileOutputStream(url.getFile()));
 		    return _tohFactory.getSerializationHandler();
                 }
                 else if (systemId.startsWith("http:")) {
                     url = new URL(systemId);
                     final URLConnection connection = url.openConnection();
-		    _tohFactory.setOutputStream(connection.getOutputStream());
+		    _tohFactory.setOutputStream(_ostream = connection.getOutputStream());
 		    return _tohFactory.getSerializationHandler();
                 }
                 else {
                     // system id is just a filename
                     url = new File(systemId).toURL();
 		    _tohFactory.setOutputStream(
-		        new FileOutputStream(url.getFile()));
+		        _ostream = new FileOutputStream(url.getFile()));
 		    return _tohFactory.getSerializationHandler();
                 }
 	    }
@@ -467,6 +492,14 @@ public final class TransformerImpl extends Transformer
     protected TransformerFactoryImpl getTransformerFactory() {
         return _tfactory;
     }
+    
+    /**
+     * Returns the {@link org.apache.xalan.xsltc.runtime.output.TransletOutputHandlerFactory}
+     * object that create the <code>TransletOutputHandler</code>.
+     */
+    protected TransletOutputHandlerFactory getTransletOutputHandlerFactory() {
+        return _tohFactory;
+    }
 
     private void transformIdentity(Source source, SerializationHandler handler)
 	throws Exception 
@@ -608,6 +641,15 @@ public final class TransformerImpl extends Transformer
 	} finally {
             _dtmManager = null;
         }
+
+	// If we create an output stream for the Result, we need to close it after the transformation.
+	if (_ostream != null) {
+	    try {
+	        _ostream.close();
+	    }
+	    catch (IOException e) {}
+	    _ostream = null;
+	}
     }
 
     /**
@@ -1140,7 +1182,7 @@ public final class TransformerImpl extends Transformer
      * invoking this method. It should still be possible for the application
      * to process the document through to the end.
      *
-     * @param exception The warning information encapsulated in a transformer 
+     * @param e The warning information encapsulated in a transformer 
      * exception.
      * @throws TransformerException if the application chooses to discontinue
      * the transformation (always does in our case).
@@ -1168,7 +1210,7 @@ public final class TransformerImpl extends Transformer
      * Transformers are free to stop reporting events once this method has
      * been invoked.
      *
-     * @param exception The warning information encapsulated in a transformer
+     * @param e The warning information encapsulated in a transformer
      * exception.
      * @throws TransformerException if the application chooses to discontinue
      * the transformation (always does in our case).
@@ -1196,7 +1238,7 @@ public final class TransformerImpl extends Transformer
      * transformation. It should still be possible for the application to
      * process the document through to the end.
      *
-     * @param exception The warning information encapsulated in a transformer
+     * @param e The warning information encapsulated in a transformer
      * exception.
      * @throws TransformerException if the application chooses to discontinue
      * the transformation (never does in our case).
@@ -1215,4 +1257,23 @@ public final class TransformerImpl extends Transformer
         }
     }
 
+    /**
+     * This method resets  the Transformer to its original configuration
+     * Transformer code is reset to the same state it was when it was
+     * created
+     * @since 1.5
+     */
+    public void reset() {
+
+        _method = null;
+        _encoding = null;
+        _sourceSystemId = null;
+        _errorListener = this;
+        _uriResolver = null;
+        _dom = null;
+        _parameters = null;
+        _indentNumber = 0;
+        setOutputProperties (null);
+
+    }
 }
