@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: XMLErrorResources_zh_CN.java,v 1.4 2004/02/17 04:14:26 minchau Exp $
+ * $Id: XMLErrorResources_zh_CN.java,v 1.5 2004/12/15 17:35:51 jycli Exp $
  */
 package org.apache.xml.res;
 
@@ -175,8 +175,14 @@ public class XMLErrorResources_zh_CN extends ListResourceBundle
 
   // Error messages...
 
-  /** The lookup table for error messages.   */
-  public static final Object[][] contents = {
+  /**
+   * Get the lookup table for error messages
+   *
+   * @return The association list.
+   */
+  public Object[][] getContents()
+  {
+    return new Object[][] {
 
   /** Error message ID that has a null message, but takes in a single object.    */
     {"ER0000" , "{0}" },
@@ -430,15 +436,6 @@ public class XMLErrorResources_zh_CN extends ListResourceBundle
 
 
   };
-
-  /**
-   * Get the association list.
-   *
-   * @return The association list.
-   */
-  public Object[][] getContents()
-  {
-    return contents;
   }
 
   /**
