@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: ToHTMLStream.java,v 1.28 2004/02/18 22:57:44 minchau Exp $
+ * $Id: ToHTMLStream.java,v 1.43 2005/08/03 19:20:31 minchau Exp $
  */
 package org.apache.xml.serializer;
 
@@ -25,18 +25,22 @@ import java.util.Properties;
 
 import javax.xml.transform.Result;
 
-import org.apache.xml.res.XMLErrorResources;
-import org.apache.xml.res.XMLMessages;
-import org.apache.xml.utils.BoolStack;
-import org.apache.xml.utils.Trie;
+import org.apache.xml.serializer.utils.MsgKey;
+import org.apache.xml.serializer.utils.Utils;
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
 
 /**
- * @author Santiago Pericas-Geertsen
- * @author G. Todd Miller 
+ * This serializer takes a series of SAX or
+ * SAX-like events and writes its output
+ * to the given stream.
+ * 
+ * This class is not a public API, it is public
+ * because it is used from another package.
+ * 
+ * @xsl.usage internal
  */
-public class ToHTMLStream extends ToStream 
+public final class ToHTMLStream extends ToStream 
 {
 
     /** This flag is set while receiving events from the DTD */
@@ -50,7 +54,7 @@ public class ToHTMLStream extends ToStream
      * Map that tells which XML characters should have special treatment, and it
      *  provides character to entity name lookup.
      */
-    protected static final CharInfo m_htmlcharInfo =
+    private static final CharInfo m_htmlcharInfo =
 //        new CharInfo(CharInfo.HTML_ENTITIES_RESOURCE);
         CharInfo.getCharInfo(CharInfo.HTML_ENTITIES_RESOURCE, Method.HTML);
 
@@ -58,6 +62,9 @@ public class ToHTMLStream extends ToStream
     static final Trie m_elementFlags = new Trie();
 
     static {
+        initTagReference(m_elementFlags);
+    }
+    static void initTagReference(Trie m_elementFlags) {
 
         // HTML 4.0 loose DTD
         m_elementFlags.put("BASEFONT", new ElemDesc(0 | ElemDesc.EMPTY));
@@ -314,7 +321,8 @@ public class ToHTMLStream extends ToStream
                     | ElemDesc.BLOCK
                     | ElemDesc.BLOCKFORM
                     | ElemDesc.BLOCKFORMFIELDSET));
-        // NS4 extensions
+                    
+        // Netscape 4 extension
         m_elementFlags.put(
             "LAYER",
             new ElemDesc(
@@ -322,6 +330,7 @@ public class ToHTMLStream extends ToStream
                     | ElemDesc.BLOCK
                     | ElemDesc.BLOCKFORM
                     | ElemDesc.BLOCKFORMFIELDSET));
+        // Netscape 4 extension                    
         m_elementFlags.put(
             "ILAYER",
             new ElemDesc(
@@ -330,127 +339,177 @@ public class ToHTMLStream extends ToStream
                     | ElemDesc.BLOCKFORM
                     | ElemDesc.BLOCKFORMFIELDSET));
 
+
+        // NOW FOR ATTRIBUTE INFORMATION . . .
         ElemDesc elemDesc;
 
-        elemDesc = (ElemDesc) m_elementFlags.get("AREA");
 
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("A");
+        elemDesc.setAttr("HREF", ElemDesc.ATTRURL);
+        elemDesc.setAttr("NAME", ElemDesc.ATTRURL);
+        
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("AREA");
         elemDesc.setAttr("HREF", ElemDesc.ATTRURL);
         elemDesc.setAttr("NOHREF", ElemDesc.ATTREMPTY);
 
+        // ----------------------------------------------
         elemDesc = (ElemDesc) m_elementFlags.get("BASE");
-
         elemDesc.setAttr("HREF", ElemDesc.ATTRURL);
 
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("BUTTON");
+        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
+
+        // ----------------------------------------------
         elemDesc = (ElemDesc) m_elementFlags.get("BLOCKQUOTE");
-
-        elemDesc.setAttr("CITE", ElemDesc.ATTRURL);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("Q");
-
-        elemDesc.setAttr("CITE", ElemDesc.ATTRURL);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("INS");
-
         elemDesc.setAttr("CITE", ElemDesc.ATTRURL);
 
+        // ----------------------------------------------
         elemDesc = (ElemDesc) m_elementFlags.get("DEL");
-
         elemDesc.setAttr("CITE", ElemDesc.ATTRURL);
+     
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("DIR");
+        elemDesc.setAttr("COMPACT", ElemDesc.ATTREMPTY);
+           
+        // ----------------------------------------------
+        
+        elemDesc = (ElemDesc) m_elementFlags.get("DIV");
+        elemDesc.setAttr("SRC", ElemDesc.ATTRURL); // Netscape 4 extension
+        elemDesc.setAttr("NOWRAP", ElemDesc.ATTREMPTY); // Internet-Explorer extension
+   
+        // ----------------------------------------------        
+        elemDesc = (ElemDesc) m_elementFlags.get("DL");
+        elemDesc.setAttr("COMPACT", ElemDesc.ATTREMPTY);
+           
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("FORM");
+        elemDesc.setAttr("ACTION", ElemDesc.ATTRURL);
 
-        elemDesc = (ElemDesc) m_elementFlags.get("A");
+        // ----------------------------------------------
+        // Attribution to: "Voytenko, Dimitry" <DVoytenko@SECTORBASE.COM>
+        elemDesc = (ElemDesc) m_elementFlags.get("FRAME");
+        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
+        elemDesc.setAttr("LONGDESC", ElemDesc.ATTRURL);
+        elemDesc.setAttr("NORESIZE",ElemDesc.ATTREMPTY);
 
-        elemDesc.setAttr("HREF", ElemDesc.ATTRURL);
-        elemDesc.setAttr("NAME", ElemDesc.ATTRURL);
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("HEAD");
+        elemDesc.setAttr("PROFILE", ElemDesc.ATTRURL);
 
-        elemDesc = (ElemDesc) m_elementFlags.get("LINK");
-        elemDesc.setAttr("HREF", ElemDesc.ATTRURL);
+        // ----------------------------------------------        
+        elemDesc = (ElemDesc) m_elementFlags.get("HR");
+        elemDesc.setAttr("NOSHADE", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------
+        // HTML 4.0, section 16.5
+        elemDesc = (ElemDesc) m_elementFlags.get("IFRAME");
+        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
+        elemDesc.setAttr("LONGDESC", ElemDesc.ATTRURL);
 
+        // ----------------------------------------------
+        // Netscape 4 extension
+        elemDesc = (ElemDesc) m_elementFlags.get("ILAYER");
+        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
+
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("IMG");
+        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
+        elemDesc.setAttr("LONGDESC", ElemDesc.ATTRURL);
+        elemDesc.setAttr("USEMAP", ElemDesc.ATTRURL);
+        elemDesc.setAttr("ISMAP", ElemDesc.ATTREMPTY);
+
+        // ----------------------------------------------
         elemDesc = (ElemDesc) m_elementFlags.get("INPUT");
-
         elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
         elemDesc.setAttr("USEMAP", ElemDesc.ATTRURL);
         elemDesc.setAttr("CHECKED", ElemDesc.ATTREMPTY);
         elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
         elemDesc.setAttr("ISMAP", ElemDesc.ATTREMPTY);
         elemDesc.setAttr("READONLY", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("INS");
+        elemDesc.setAttr("CITE", ElemDesc.ATTRURL);
 
-        elemDesc = (ElemDesc) m_elementFlags.get("SELECT");
-
-        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
-        elemDesc.setAttr("MULTIPLE", ElemDesc.ATTREMPTY);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("OPTGROUP");
-
-        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("OPTION");
-
-        elemDesc.setAttr("SELECTED", ElemDesc.ATTREMPTY);
-        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("TEXTAREA");
-
-        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
-        elemDesc.setAttr("READONLY", ElemDesc.ATTREMPTY);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("BUTTON");
-
-        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("SCRIPT");
-
+        // ----------------------------------------------
+        // Netscape 4 extension
+        elemDesc = (ElemDesc) m_elementFlags.get("LAYER");
         elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
-        elemDesc.setAttr("FOR", ElemDesc.ATTRURL);
-        elemDesc.setAttr("DEFER", ElemDesc.ATTREMPTY);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("IMG");
-
-        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
-        elemDesc.setAttr("LONGDESC", ElemDesc.ATTRURL);
-        elemDesc.setAttr("USEMAP", ElemDesc.ATTRURL);
-        elemDesc.setAttr("ISMAP", ElemDesc.ATTREMPTY);
 
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("LINK");
+        elemDesc.setAttr("HREF", ElemDesc.ATTRURL);
+       
+        // ----------------------------------------------       
+        elemDesc = (ElemDesc) m_elementFlags.get("MENU");
+        elemDesc.setAttr("COMPACT", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------
         elemDesc = (ElemDesc) m_elementFlags.get("OBJECT");
-
         elemDesc.setAttr("CLASSID", ElemDesc.ATTRURL);
         elemDesc.setAttr("CODEBASE", ElemDesc.ATTRURL);
         elemDesc.setAttr("DATA", ElemDesc.ATTRURL);
         elemDesc.setAttr("ARCHIVE", ElemDesc.ATTRURL);
         elemDesc.setAttr("USEMAP", ElemDesc.ATTRURL);
         elemDesc.setAttr("DECLARE", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------        
+        elemDesc = (ElemDesc) m_elementFlags.get("OL");
+        elemDesc.setAttr("COMPACT", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("OPTGROUP");
+        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
 
-        elemDesc = (ElemDesc) m_elementFlags.get("FORM");
-
-        elemDesc.setAttr("ACTION", ElemDesc.ATTRURL);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("HEAD");
-
-        elemDesc.setAttr("PROFILE", ElemDesc.ATTRURL);
-
-        // Attribution to: "Voytenko, Dimitry" <DVoytenko@SECTORBASE.COM>
-        elemDesc = (ElemDesc) m_elementFlags.get("FRAME");
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("OPTION");
+        elemDesc.setAttr("SELECTED", ElemDesc.ATTREMPTY);
+        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("Q");
+        elemDesc.setAttr("CITE", ElemDesc.ATTRURL);
 
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("SCRIPT");
         elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
-        elemDesc.setAttr("LONGDESC", ElemDesc.ATTRURL);
+        elemDesc.setAttr("FOR", ElemDesc.ATTRURL);
+        elemDesc.setAttr("DEFER", ElemDesc.ATTREMPTY);
 
-        // HTML 4.0, section 16.5
-        elemDesc = (ElemDesc) m_elementFlags.get("IFRAME");
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("SELECT");
+        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
+        elemDesc.setAttr("MULTIPLE", ElemDesc.ATTREMPTY);
 
-        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
-        elemDesc.setAttr("LONGDESC", ElemDesc.ATTRURL);
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("TABLE");
+        elemDesc.setAttr("NOWRAP", ElemDesc.ATTREMPTY); // Internet-Explorer extension
+        
+        // ----------------------------------------------        
+        elemDesc = (ElemDesc) m_elementFlags.get("TD");
+        elemDesc.setAttr("NOWRAP", ElemDesc.ATTREMPTY);
 
-        // NS4 extensions
-        elemDesc = (ElemDesc) m_elementFlags.get("LAYER");
-
-        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("ILAYER");
-
-        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
-
-        elemDesc = (ElemDesc) m_elementFlags.get("DIV");
-
-        elemDesc.setAttr("SRC", ElemDesc.ATTRURL);
+        // ----------------------------------------------
+        elemDesc = (ElemDesc) m_elementFlags.get("TEXTAREA");
+        elemDesc.setAttr("DISABLED", ElemDesc.ATTREMPTY);
+        elemDesc.setAttr("READONLY", ElemDesc.ATTREMPTY);
+       
+        // ----------------------------------------------                
+        elemDesc = (ElemDesc) m_elementFlags.get("TH");
+        elemDesc.setAttr("NOWRAP", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------
+        // The nowrap attribute of a tr element is both
+        // a Netscape and Internet-Explorer extension                
+        elemDesc = (ElemDesc) m_elementFlags.get("TR");
+        elemDesc.setAttr("NOWRAP", ElemDesc.ATTREMPTY);
+        
+        // ----------------------------------------------        
+        elemDesc = (ElemDesc) m_elementFlags.get("UL");
+        elemDesc.setAttr("COMPACT", ElemDesc.ATTREMPTY);
     }
 
     /**
@@ -490,8 +549,15 @@ public class ToHTMLStream extends ToStream
      * it will switch to the new format. This method should not be
      * called while the serializer is in the process of serializing
      * a document.
+     * 
+     * This method can be called multiple times before starting
+     * the serialization of a particular result-tree. In principle
+     * all serialization parameters can be changed, with the exception
+     * of method="html" (it must be method="html" otherwise we
+     * shouldn't even have a ToHTMLStream object here!) 
      *
-     * @param format The output format to use
+     * @param format The output format or serialzation parameters
+     * to use.
      */
     public void setOutputFormat(Properties format)
     {
@@ -547,6 +613,24 @@ public class ToHTMLStream extends ToStream
             return (ElemDesc)obj;
         return m_dummy;
     }
+    
+    /**
+     * A Trie that is just a copy of the "static" one.
+     * We need this one to be able to use the faster, but not thread-safe
+     * method Trie.get2(name)
+     */
+    private Trie m_htmlInfo = new Trie(m_elementFlags);
+    /**
+     * Calls to this method could be replaced with calls to
+     * getElemDesc(name), but this one should be faster.
+     */
+    private ElemDesc getElemDesc2(String name)
+    {
+        Object obj = m_htmlInfo.get2(name);
+        if (null != obj)
+            return (ElemDesc)obj;
+        return m_dummy;
+    }
 
     /**
      * Default constructor.
@@ -590,7 +674,7 @@ public class ToHTMLStream extends ToStream
                 final java.io.Writer writer = m_writer;
                 try
                 {
-                writer.write("<!DOCTYPE HTML");
+                writer.write("<!DOCTYPE html");
 
                 if (null != doctypePublic)
                 {
@@ -604,7 +688,7 @@ public class ToHTMLStream extends ToStream
                     if (null == doctypePublic)
                         writer.write(" SYSTEM \"");
                     else
-                        writer.write('"');
+                        writer.write(" \"");
 
                     writer.write(doctypeSystem);
                     writer.write('"');
@@ -703,7 +787,8 @@ public class ToHTMLStream extends ToStream
         
         try
         {
-            ElemDesc elemDesc = getElemDesc(name);
+            // getElemDesc2(name) is faster than getElemDesc(name)
+            ElemDesc elemDesc = getElemDesc2(name);
             int elemFlags = elemDesc.getFlags();
 
             // deal with indentation issues first
@@ -1088,7 +1173,7 @@ public class ToHTMLStream extends ToStream
                         writer.write('%');
                         writer.write(makeHHString(low));
                     }
-                    else if (isUTF16Surrogate(ch)) // high surrogate
+                    else if (Encodings.isHighUTF16Surrogate(ch)) // high surrogate
                     {
                         // I'm sure this can be done in 3 instructions, but I choose 
                         // to try and do it exactly like it is done in the book, at least 
@@ -1287,7 +1372,7 @@ public class ToHTMLStream extends ToStream
                     writer.write(chars,cleanStart,cleanLength);
                     cleanLength = 0;
                 }
-                int pos = accumDefaultEntity(writer, ch, i, chars, end, false, false);
+                int pos = accumDefaultEntity(writer, ch, i, chars, end, false, true);
 
                 if (i != pos)
                 {
@@ -1295,7 +1380,7 @@ public class ToHTMLStream extends ToStream
                 }
                 else
                 {
-                    if (isUTF16Surrogate(ch))
+                    if (Encodings.isHighUTF16Surrogate(ch))
                     {
  
                             writeUTF16Surrogate(ch, chars, i, end);
@@ -1315,12 +1400,10 @@ public class ToHTMLStream extends ToStream
                     }
                     else
                     */
-                    String entityName = m_charInfo.getEntityNameForChar(ch);
-                    if (null != entityName)
+                    String outputStringForChar = m_charInfo.getOutputStringForChar(ch);
+                    if (null != outputStringForChar)
                     {
-                        writer.write('&');
-                        writer.write(entityName);
-                        writer.write(';');
+                        writer.write(outputStringForChar);
                     }
                     else if (escapingNotNeeded(ch))
                     {
@@ -1421,8 +1504,8 @@ public class ToHTMLStream extends ToStream
             catch (IOException ioe)
             {
                 throw new org.xml.sax.SAXException(
-                    XMLMessages.createXMLMessage(
-                        XMLErrorResources.ER_OIERROR,
+                    Utils.messages.createMessage(
+                        MsgKey.ER_OIERROR,
                         null),
                     ioe);
                 //"IO error", ioe);
@@ -1488,8 +1571,8 @@ public class ToHTMLStream extends ToStream
             catch (IOException ioe)
             {
                 throw new org.xml.sax.SAXException(
-                    XMLMessages.createXMLMessage(
-                        XMLErrorResources.ER_OIERROR,
+                    Utils.messages.createMessage(
+                        MsgKey.ER_OIERROR,
                         null),
                     ioe);
                 //"IO error", ioe);
@@ -1599,7 +1682,7 @@ public class ToHTMLStream extends ToStream
         }
     }
     /**
-     * @see org.apache.xml.serializer.ExtendedContentHandler#endElement(String)
+     * @see ExtendedContentHandler#endElement(String)
      */
     public final void endElement(String elemName) throws SAXException
     {
@@ -1733,7 +1816,7 @@ public class ToHTMLStream extends ToStream
          * @param uri the URI of the namespace
          * @param prefix the prefix associated with the given URI.
          *
-         * @see org.apache.xml.serializer.ExtendedContentHandler#namespaceAfterStartElement(String, String)
+         * @see ExtendedContentHandler#namespaceAfterStartElement(String, String)
          */
         public void namespaceAfterStartElement(String prefix, String uri)
             throws SAXException
@@ -1897,4 +1980,324 @@ public class ToHTMLStream extends ToStream
         m_omitMetaTag = false;
         m_specialEscapeURLs = true;     
     }
+    
+    static class Trie
+    {
+        /**
+         * A digital search trie for 7-bit ASCII text
+         * The API is a subset of java.util.Hashtable
+         * The key must be a 7-bit ASCII string
+         * The value may be any Java Object
+         * One can get an object stored in a trie from its key, 
+         * but the search is either case sensitive or case 
+         * insensitive to the characters in the key, and this
+         * choice of sensitivity or insensitivity is made when
+         * the Trie is created, before any objects are put in it.
+         * 
+         * This class is a copy of the one in org.apache.xml.utils. 
+         * It exists to cut the serializers dependancy on that package.
+         *  
+         * @xsl.usage internal
+         */
+
+        /** Size of the m_nextChar array.  */
+        public static final int ALPHA_SIZE = 128;
+
+        /** The root node of the tree.    */
+        final Node m_Root;
+
+        /** helper buffer to convert Strings to char arrays */
+        private char[] m_charBuffer = new char[0];
+
+        /** true if the search for an object is lower case only with the key */
+        private final boolean m_lowerCaseOnly;
+
+        /**
+         * Construct the trie that has a case insensitive search.
+         */
+        public Trie()
+        {
+            m_Root = new Node();
+            m_lowerCaseOnly = false;
+        }
+
+        /**
+         * Construct the trie given the desired case sensitivity with the key.
+         * @param lowerCaseOnly true if the search keys are to be loser case only,
+         * not case insensitive.
+         */
+        public Trie(boolean lowerCaseOnly)
+        {
+            m_Root = new Node();
+            m_lowerCaseOnly = lowerCaseOnly;
+        }
+
+        /**
+         * Put an object into the trie for lookup.
+         *
+         * @param key must be a 7-bit ASCII string
+         * @param value any java object.
+         *
+         * @return The old object that matched key, or null.
+         */
+        public Object put(String key, Object value)
+        {
+
+            final int len = key.length();
+            if (len > m_charBuffer.length)
+            {
+                // make the biggest buffer ever needed in get(String)
+                m_charBuffer = new char[len];
+            }
+
+            Node node = m_Root;
+
+            for (int i = 0; i < len; i++)
+            {
+                Node nextNode =
+                    node.m_nextChar[Character.toLowerCase(key.charAt(i))];
+
+                if (nextNode != null)
+                {
+                    node = nextNode;
+                }
+                else
+                {
+                    for (; i < len; i++)
+                    {
+                        Node newNode = new Node();
+                        if (m_lowerCaseOnly)
+                        {
+                            // put this value into the tree only with a lower case key 
+                            node.m_nextChar[Character.toLowerCase(
+                                key.charAt(i))] =
+                                newNode;
+                        }
+                        else
+                        {
+                            // put this value into the tree with a case insensitive key
+                            node.m_nextChar[Character.toUpperCase(
+                                key.charAt(i))] =
+                                newNode;
+                            node.m_nextChar[Character.toLowerCase(
+                                key.charAt(i))] =
+                                newNode;
+                        }
+                        node = newNode;
+                    }
+                    break;
+                }
+            }
+
+            Object ret = node.m_Value;
+
+            node.m_Value = value;
+
+            return ret;
+        }
+
+        /**
+         * Get an object that matches the key.
+         *
+         * @param key must be a 7-bit ASCII string
+         *
+         * @return The object that matches the key, or null.
+         */
+        public Object get(final String key)
+        {
+
+            final int len = key.length();
+
+            /* If the name is too long, we won't find it, this also keeps us
+             * from overflowing m_charBuffer
+             */
+            if (m_charBuffer.length < len)
+                return null;
+
+            Node node = m_Root;
+            switch (len) // optimize the look up based on the number of chars
+            {
+                // case 0 looks silly, but the generated bytecode runs
+                // faster for lookup of elements of length 2 with this in
+                // and a fair bit faster.  Don't know why.
+                case 0 :
+                    {
+                        return null;
+                    }
+
+                case 1 :
+                    {
+                        final char ch = key.charAt(0);
+                        if (ch < ALPHA_SIZE)
+                        {
+                            node = node.m_nextChar[ch];
+                            if (node != null)
+                                return node.m_Value;
+                        }
+                        return null;
+                    }
+                    //                comment out case 2 because the default is faster            
+                    //                case 2 :
+                    //                    {
+                    //                        final char ch0 = key.charAt(0);
+                    //                        final char ch1 = key.charAt(1);
+                    //                        if (ch0 < ALPHA_SIZE && ch1 < ALPHA_SIZE)
+                    //                        {
+                    //                            node = node.m_nextChar[ch0];
+                    //                            if (node != null)
+                    //                            {
+                    //                        
+                    //                                if (ch1 < ALPHA_SIZE) 
+                    //                                {
+                    //                                    node = node.m_nextChar[ch1];
+                    //                                    if (node != null)
+                    //                                        return node.m_Value;
+                    //                                }
+                    //                            }
+                    //                        }
+                    //                        return null;
+                    //                   }
+                default :
+                    {
+                        for (int i = 0; i < len; i++)
+                        {
+                            // A thread-safe way to loop over the characters
+                            final char ch = key.charAt(i);
+                            if (ALPHA_SIZE <= ch)
+                            {
+                                // the key is not 7-bit ASCII so we won't find it here
+                                return null;
+                            }
+
+                            node = node.m_nextChar[ch];
+                            if (node == null)
+                                return null;
+                        }
+
+                        return node.m_Value;
+                    }
+            }
+        }
+
+        /**
+         * The node representation for the trie.
+         * @xsl.usage internal
+         */
+        private class Node
+        {
+
+            /**
+             * Constructor, creates a Node[ALPHA_SIZE].
+             */
+            Node()
+            {
+                m_nextChar = new Node[ALPHA_SIZE];
+                m_Value = null;
+            }
+
+            /** The next nodes.   */
+            final Node m_nextChar[];
+
+            /** The value.   */
+            Object m_Value;
+        }
+        /**
+         * Construct the trie from another Trie.
+         * Both the existing Trie and this new one share the same table for
+         * lookup, and it is assumed that the table is fully populated and
+         * not changing anymore.
+         * 
+         * @param existingTrie the Trie that this one is a copy of.
+         */
+        public Trie(Trie existingTrie)
+        {
+            // copy some fields from the existing Trie into this one.
+            m_Root = existingTrie.m_Root;
+            m_lowerCaseOnly = existingTrie.m_lowerCaseOnly;
+
+            // get a buffer just big enough to hold the longest key in the table.
+            int max = existingTrie.getLongestKeyLength();
+            m_charBuffer = new char[max];
+        }
+
+        /**
+         * Get an object that matches the key.
+         * This method is faster than get(), but is not thread-safe.
+         *
+         * @param key must be a 7-bit ASCII string
+         *
+         * @return The object that matches the key, or null.
+         */
+        public Object get2(final String key)
+        {
+
+            final int len = key.length();
+
+            /* If the name is too long, we won't find it, this also keeps us
+             * from overflowing m_charBuffer
+             */
+            if (m_charBuffer.length < len)
+                return null;
+
+            Node node = m_Root;
+            switch (len) // optimize the look up based on the number of chars
+            {
+                // case 0 looks silly, but the generated bytecode runs
+                // faster for lookup of elements of length 2 with this in
+                // and a fair bit faster.  Don't know why.
+                case 0 :
+                    {
+                        return null;
+                    }
+
+                case 1 :
+                    {
+                        final char ch = key.charAt(0);
+                        if (ch < ALPHA_SIZE)
+                        {
+                            node = node.m_nextChar[ch];
+                            if (node != null)
+                                return node.m_Value;
+                        }
+                        return null;
+                    }
+                default :
+                    {
+                        /* Copy string into array. This is not thread-safe because
+                         * it modifies the contents of m_charBuffer. If multiple
+                         * threads were to use this Trie they all would be
+                         * using this same array (not good). So this 
+                         * method is not thread-safe, but it is faster because
+                         * converting to a char[] and looping over elements of
+                         * the array is faster than a String's charAt(i).
+                         */
+                        key.getChars(0, len, m_charBuffer, 0);
+
+                        for (int i = 0; i < len; i++)
+                        {
+                            final char ch = m_charBuffer[i];
+                            if (ALPHA_SIZE <= ch)
+                            {
+                                // the key is not 7-bit ASCII so we won't find it here
+                                return null;
+                            }
+
+                            node = node.m_nextChar[ch];
+                            if (node == null)
+                                return null;
+                        }
+
+                        return node.m_Value;
+                    }
+            }
+        }
+
+        /**
+         * Get the length of the longest key used in the table. 
+         */
+        public int getLongestKeyLength()
+        {
+            return m_charBuffer.length;
+        }
+    }
 }
