@@ -14,21 +14,25 @@
  * limitations under the License.
  */
 /*
- * $Id: ToTextStream.java,v 1.15 2004/02/17 04:18:18 minchau Exp $
+ * $Id: ToTextStream.java,v 1.22 2005/08/03 19:20:31 minchau Exp $
  */
 package org.apache.xml.serializer;
 
 import java.io.IOException;
 
-import org.apache.xml.res.XMLErrorResources;
-import org.apache.xml.res.XMLMessages;
+import org.apache.xml.serializer.utils.MsgKey;
+import org.apache.xml.serializer.utils.Utils;
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
 
 /**
- * @author Santiago Pericas-Geertsen
+ * This class is not a public API.
+ * It is only public because it is used in other packages. 
+ * This class converts SAX or SAX-like calls to a 
+ * serialized document for xsl:output method of "text".
+ * @xsl.usage internal
  */
-public class ToTextStream extends ToStream 
+public final class ToTextStream extends ToStream 
 { 
 
        
@@ -149,7 +153,6 @@ public class ToTextStream extends ToStream
    *        performed.
    * @param name The qualified name (with prefix), or the
    *        empty string if qualified names are not available.
-   * @param name The element type name
    * @throws org.xml.sax.SAXException Any SAX exception, possibly
    *            wrapping another exception.
    *
@@ -191,12 +194,29 @@ public class ToTextStream extends ToStream
           throws org.xml.sax.SAXException
   {
 
-    // this.accum(ch, start, length);
     flushPending();    
     
     try
     {
-        writeNormalizedChars(ch, start, length, false, m_lineSepUse);
+        if (inTemporaryOutputState()) {
+            /* leave characters un-processed as we are
+             * creating temporary output, the output generated by
+             * this serializer will be input to a final serializer 
+             * later on and it will do the processing in final
+             * output state (not temporary output state).
+             * 
+             * A "temporary" ToTextStream serializer is used to
+             * evaluate attribute value templates (for example),
+             * and the result of evaluating such a thing
+             * is fed into a final serializer later on.
+             */
+            m_writer.write(ch, start, length);
+        }
+        else {
+            // In final output state we do process the characters!
+            writeNormalizedChars(ch, start, length, m_lineSepUse);
+        }
+            
         if (m_tracer != null)
             super.fireCharEvent(ch, start, length);      
     }
@@ -223,7 +243,7 @@ public class ToTextStream extends ToStream
 
     try
     {
-      writeNormalizedChars(ch, start, length, false, m_lineSepUse);
+      writeNormalizedChars(ch, start, length, m_lineSepUse);
     }
     catch(IOException ioe)
     {
@@ -231,168 +251,101 @@ public class ToTextStream extends ToStream
     }
   }
   
-/**
- * Normalize the characters, but don't escape.  Different from 
- * SerializerToXML#writeNormalizedChars because it does not attempt to do 
- * XML escaping at all.
- *
- * @param ch The characters from the XML document.
- * @param start The start position in the array.
- * @param length The number of characters to read from the array.
- * @param isCData true if a CDATA block should be built around the characters.
- * @param useLineSep true if the operating systems 
- * end-of-line separator should be output rather than a new-line character.
- * 
- * @throws IOException
- * @throws org.xml.sax.SAXException
- */
-void writeNormalizedChars(
-    final char ch[],
-    final int start,
-    final int length,
-    final boolean isCData,
-    final boolean useLineSep)
-    throws IOException, org.xml.sax.SAXException
-{
-    final java.io.Writer writer = m_writer;
-    final int end = start + length;
-
-    /* copy a few "constants" before the loop for performance */
-    final char S_LINEFEED = CharInfo.S_LINEFEED;
-    final int M_MAXCHARACTER = this.m_maxCharacter;
-
-    if (isCData)
+    /**
+     * Normalize the characters, but don't escape.  Different from 
+     * SerializerToXML#writeNormalizedChars because it does not attempt to do 
+     * XML escaping at all.
+     *
+     * @param ch The characters from the XML document.
+     * @param start The start position in the array.
+     * @param length The number of characters to read from the array.
+     * @param useLineSep true if the operating systems 
+     * end-of-line separator should be output rather than a new-line character.
+     * 
+     * @throws IOException
+     * @throws org.xml.sax.SAXException
+     */
+    void writeNormalizedChars(
+        final char ch[],
+            final int start,
+            final int length,
+            final boolean useLineSep)
+            throws IOException, org.xml.sax.SAXException 
     {
+        final String encoding = getEncoding();
+        final java.io.Writer writer = m_writer;
+        final int end = start + length;
+
+        /* copy a few "constants" before the loop for performance */
+        final char S_LINEFEED = CharInfo.S_LINEFEED;
+
         // This for() loop always increments i by one at the end
         // of the loop.  Additional increments of i adjust for when
-        // two input characters are processed.
-        for (int i = start; i < end; i++)
-        {
+        // two input characters (a high/low UTF16 surrogate pair)
+        // are processed.
+        for (int i = start; i < end; i++) {
             final char c = ch[i];
 
-            if (S_LINEFEED == c && useLineSep)
-            {
+            if (S_LINEFEED == c && useLineSep) {
                 writer.write(m_lineSep, 0, m_lineSepLen);
-            }
-            else if (c > M_MAXCHARACTER)
-            {
-                if (i != 0)
-                    closeCDATA();
-
-                // This needs to go into a function...
-                if (isUTF16Surrogate(c))
-                {
-                    writeUTF16Surrogate(c, ch, i, end);
-                    i++; // two input characters processed
-                }
-                else
-                {
-                    writer.write(c);
-                }
-
-                if ((i != 0) && (i < (end - 1)))
-                {
-                    writer.write(CDATA_DELIMITER_OPEN);
-                    m_cdataTagOpen = true;
-                }
-            }
-            else if (
-                ((i < (end - 2))
-                    && (']' == c)
-                    && (']' == ch[i + 1])
-                    && ('>' == ch[i + 2])))
-            {
-                writer.write(CDATA_CONTINUE);
-                i += 2;
-            }
-            else
-            {
-                if (c <= M_MAXCHARACTER)
-                {
-                    writer.write(c);
-                }
-
-                else if (isUTF16Surrogate(c))
-                {
-                    writeUTF16Surrogate(c, ch, i, end);
-                    i++; // two input characters processed
-                }
-                else
-                {
-                    /* The character is greater than the allowed 
-                     * maximum value and it is not part of a UTF-16
-                     * pair that would be put out as a character reference.
-                     */
-                    String encoding = getEncoding();
-                    if (encoding != null)
-                    {
-                        /* The output encoding is known, 
-                         * so somthing is wrong.
-                         */ 
-                        String integralValue = Integer.toString(c);
-                        throw new SAXException(XMLMessages.createXMLMessage(
-                            XMLErrorResources.ER_ILLEGAL_CHARACTER,
-                            new Object[]{ integralValue, encoding}));
-                    }
-                    else 
-                    {
-                        /* The output encoding is not known,
-                         * so just write it out as-is.
-                         */                        
-                        writer.write(c);
-                    }
-                }
-            }
-        }
-    }
-    else
-    {
-        // not in CDATA section
-        for (int i = start; i < end; i++)
-        {
-            final char c = ch[i];
-
-            if (S_LINEFEED == c && useLineSep)
-            {
-                writer.write(m_lineSep, 0, m_lineSepLen);
-            }
-            else if (c <= M_MAXCHARACTER)
-            {
+                // one input char processed
+            } else if (m_encodingInfo.isInEncoding(c)) {
                 writer.write(c);
-            }
-            else if (isUTF16Surrogate(c))
-            {
-                writeUTF16Surrogate(c, ch, i, end);
-                i++; // two input characters processed
-            }
-            else
-            {
-                /* The character is greater than the allowed 
-                 * maximum value and it is not part of a UTF-16
-                 * pair that would be put out as a character reference.
-                 */
-                String encoding = getEncoding();
-                if (encoding != null) 
-                {
+                // one input char processed    
+            } else if (Encodings.isHighUTF16Surrogate(c)) {
+                final int codePoint = writeUTF16Surrogate(c, ch, i, end);
+                if (codePoint != 0) {
+                    // I think we can just emit the message,
+                    // not crash and burn.
+                    final String integralValue = Integer.toString(codePoint);
+                    final String msg = Utils.messages.createMessage(
+                        MsgKey.ER_ILLEGAL_CHARACTER,
+                        new Object[] { integralValue, encoding });
+                      
+                    //Older behavior was to throw the message,
+                    //but newer gentler behavior is to write a message to System.err
+                    //throw new SAXException(msg);
+                    System.err.println(msg);                            
+
+                }
+                i++; // two input chars processed               
+            } else {
+                // Don't know what to do with this char, it is
+                // not in the encoding and not a high char in
+                // a surrogate pair, so write out as an entity ref
+                if (encoding != null) {
                     /* The output encoding is known, 
                      * so somthing is wrong.
-                     */ 
-                    String integralValue = Integer.toString(c);
-                    throw new SAXException(XMLMessages.createXMLMessage(
-                        XMLErrorResources.ER_ILLEGAL_CHARACTER,
-                        new Object[]{ integralValue, encoding}));
-                }
-                else 
-                {
+                     */
+
+                    // not in the encoding, so write out a character reference
+                    writer.write('&');
+                    writer.write('#');
+                    writer.write(Integer.toString(c));
+                    writer.write(';');
+
+                    // I think we can just emit the message,
+                    // not crash and burn.
+                    final String integralValue = Integer.toString(c);
+                    final String msg = Utils.messages.createMessage(
+                        MsgKey.ER_ILLEGAL_CHARACTER,
+                        new Object[] { integralValue, encoding });
+                      
+                    //Older behavior was to throw the message,
+                    //but newer gentler behavior is to write a message to System.err
+                    //throw new SAXException(msg);
+                    System.err.println(msg); 
+                } else {
                     /* The output encoding is not known,
                      * so just write it out as-is.
-                     */                        
+                     */
                     writer.write(c);
-                }                
+                }
+
+                // one input char was processed
             }
         }
     }
-}
 
   /**
    * Receive notification of cdata.
@@ -424,7 +377,7 @@ void writeNormalizedChars(
   {
     try
     {
-        writeNormalizedChars(ch, start, length, false, m_lineSepUse);
+        writeNormalizedChars(ch, start, length, m_lineSepUse);
         if (m_tracer != null)
             super.fireCDATAEvent(ch, start, length);              
     }
@@ -466,7 +419,7 @@ void writeNormalizedChars(
 
     try
     {
-      writeNormalizedChars(ch, start, length, false, m_lineSepUse);
+      writeNormalizedChars(ch, start, length, m_lineSepUse);
     }
     catch(IOException ioe)
     {
@@ -558,14 +511,15 @@ void writeNormalizedChars(
   }
   
     /**
-     * @see org.apache.xml.serializer.ExtendedContentHandler#addAttribute(String, String, String, String, String)
+     * @see ExtendedContentHandler#addAttribute(String, String, String, String, String)
      */
     public void addAttribute(
         String uri,
         String localName,
         String rawName,
         String type,
-        String value)
+        String value,
+        boolean XSLAttribute)
     {
         // do nothing, just forget all about the attribute
     }
@@ -579,7 +533,7 @@ void writeNormalizedChars(
     }
 
     /**
-     * @see org.apache.xml.serializer.ExtendedContentHandler#endElement(String)
+     * @see ExtendedContentHandler#endElement(String)
      */
     public void endElement(String elemName) throws SAXException
     {
