@@ -49,7 +49,7 @@ public class XMLChar {
     //
 
     /** Character flags. */
-    public static final byte[] CHARS = new byte[1 << 16];
+    private static final byte[] CHARS = new byte[1 << 16];
 
     /** Valid character mask. */
     public static final int MASK_VALID = 0x01;
@@ -539,7 +539,7 @@ public class XMLChar {
      * Check to see if a string is a valid NCName according to [4]
      * from the XML Namespaces 1.0 Recommendation
      *
-     * @param name string to check
+     * @param ncName string to check
      * @return true if name is a valid NCName
      */
     public static boolean isValidNCName(String ncName) {
