@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: StepPattern.java,v 1.28 2004/02/17 04:35:37 minchau Exp $
+ * $Id: StepPattern.java,v 1.31 2005/01/23 01:08:22 mcnamara Exp $
  */
 package org.apache.xpath.patterns;
 
@@ -36,6 +36,7 @@ import org.apache.xpath.objects.XObject;
  */
 public class StepPattern extends NodeTest implements SubContextList, ExpressionOwner
 {
+    static final long serialVersionUID = 9071668960168152644L;
 
   /** The axis for this test. */
   protected int m_axis;
@@ -791,7 +792,7 @@ public class StepPattern extends NodeTest implements SubContextList, ExpressionO
       if (pat != this)
         buf.append("/");
 
-      buf.append(Axis.names[pat.m_axis]);
+      buf.append(Axis.getNames(pat.m_axis));
       buf.append("::");
 
       if (0x000005000 == pat.m_whatToShow)
@@ -959,7 +960,7 @@ public class StepPattern extends NodeTest implements SubContextList, ExpressionO
   }
   
   /**
-   * @see XPathVisitable#callVisitors(ExpressionOwner, XPathVisitor)
+   * @see org.apache.xpath.XPathVisitable#callVisitors(ExpressionOwner, XPathVisitor)
    */
   public void callVisitors(ExpressionOwner owner, XPathVisitor visitor)
   {
