@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 /*
- * $Id: XPATHErrorResources_ja.java,v 1.8 2004/02/17 04:36:46 minchau Exp $
+ * $Id: XPATHErrorResources_ja.java,v 1.10 2005/01/23 01:39:54 mcnamara Exp $
  */
 package org.apache.xpath.res;
 
@@ -351,7 +351,14 @@ public static final String ER_IGNORABLE_WHITESPACE_NOT_HANDLED =
 
   // Error messages...
 
-  public static final Object[][] contents = {
+  /**
+   * Get the association list.
+   *
+   * @return The association list.
+   */
+  public Object[][] getContents()
+  {
+    return new Object[][]{
 
   /** Field ERROR0000          */
 
@@ -1222,6 +1229,7 @@ public static final String ER_IGNORABLE_WHITESPACE_NOT_HANDLED =
   { "three", "3" }
 
   };
+  }
 
 
   // ================= INFRASTRUCTURE ======================
@@ -1254,23 +1262,11 @@ public static final String ER_IGNORABLE_WHITESPACE_NOT_HANDLED =
   /** Field QUERY_HEADER          */
   public static final String QUERY_HEADER = "\u30d1\u30bf\u30fc\u30f3 ";
 
-  /**
-   * Get the association list.
-   *
-   * @return The association list.
-   */
-  public Object[][] getContents()
-  {
-    return contents;
-  }
 
   /**
    * Return a named ResourceBundle for a particular locale.  This method mimics the behavior
    * of ResourceBundle.getBundle().
    *
-   * @param res the name of the resource to load.
-   * @param locale the locale to prefer when searching for the bundle
-   *
    * @param className Name of local-specific subclass.
    * @return the ResourceBundle
    * @throws MissingResourceException
